Package: maps
Title: Draw Geographical Maps
Version: 3.4.3
Date: 2025-05-15
Description: Display of maps.  Projection code and larger maps are in
             separate packages ('mapproj' and 'mapdata').
Depends: R (>= 3.5.0)
Imports: graphics, utils
LazyData: yes
Suggests: mapproj (>= 1.2-0), mapdata (>= 2.3.0), sf, rnaturalearth
License: GPL-2
URL: https://github.com/adeckmyn/maps
BugReports: https://github.com/adeckmyn/maps/issues
Authors@R: c(person(given  = c("Richard", "A."),
                    family = "Becker",
                    role   = "aut",
                    comment= "Original S code"),
             person(given  = c("Allan", "R."),
                    family = "Wilks",
                    role   = "aut",
                    comment= "Original S code"),
             person(given  = "Ray",
                    family = "Brownrigg",
                    role   = c("trl", "aut"),
                    comment= "R version"),
             person(given  = c("Thomas", "P."),
                    family = "Minka",
                    role   = "aut",
                    comment= "Enhancements"),
             person(given  = "Alex",
                    family = "Deckmyn",
                    role   = c("aut", "cre"),
                    comment= c(ORCID="0009-0002-2010-8310"),
                    email  = "alex.deckmyn@meteo.be"))
NeedsCompilation: yes
Repository: CRAN
Packaged: 2025-05-19 08:35:55 UTC; alex
Author: Richard A. Becker [aut] (Original S code),
  Allan R. Wilks [aut] (Original S code),
  Ray Brownrigg [trl, aut] (R version),
  Thomas P. Minka [aut] (Enhancements),
  Alex Deckmyn [aut, cre] (ORCID:
    <https://orcid.org/0009-0002-2010-8310>)
Maintainer: Alex Deckmyn <alex.deckmyn@meteo.be>
Date/Publication: 2025-05-26 17:40:02 UTC
Built: R 4.4.0; amd64-portbld-midnightbsd4.0.2; 2026-02-17 21:05:07 UTC; unix

Package: maps
Title: Draw Geographical Maps
Version: 3.4.3
Date: 2025-05-15
Description: Display of maps.  Projection code and larger maps are in
             separate packages ('mapproj' and 'mapdata').
Depends: R (>= 3.5.0)
Imports: graphics, utils
LazyData: yes
Suggests: mapproj (>= 1.2-0), mapdata (>= 2.3.0), sf, rnaturalearth
License: GPL-2
URL: https://github.com/adeckmyn/maps
BugReports: https://github.com/adeckmyn/maps/issues
Authors@R: c(person(given  = c("Richard", "A."),
                    family = "Becker",
                    role   = "aut",
                    comment= "Original S code"),
             person(given  = c("Allan", "R."),
                    family = "Wilks",
                    role   = "aut",
                    comment= "Original S code"),
             person(given  = "Ray",
                    family = "Brownrigg",
                    role   = c("trl", "aut"),
                    comment= "R version"),
             person(given  = c("Thomas", "P."),
                    family = "Minka",
                    role   = "aut",
                    comment= "Enhancements"),
             person(given  = "Alex",
                    family = "Deckmyn",
                    role   = c("aut", "cre"),
                    comment= c(ORCID="0009-0002-2010-8310"),
                    email  = "alex.deckmyn@meteo.be"))
NeedsCompilation: yes
Repository: CRAN
Packaged: 2025-05-19 08:35:55 UTC; alex
Author: Richard A. Becker [aut] (Original S code),
  Allan R. Wilks [aut] (Original S code),
  Ray Brownrigg [trl, aut] (R version),
  Thomas P. Minka [aut] (Enhancements),
  Alex Deckmyn [aut, cre] (ORCID:
    <https://orcid.org/0009-0002-2010-8310>)
Maintainer: Alex Deckmyn <alex.deckmyn@meteo.be>
Date/Publication: 2025-05-26 17:40:02 UTC

import("graphics")
importFrom("utils", "read.delim")
export(iso.expand,sov.expand,iso.alpha,area.map, identify.map, 
       map, map.axes, map.cities, map.scale, map.text, map.where, map.wrap, 
       match.map, smooth.map, .map.range,
       SpatialPolygons2map, SpatialLines2map, map.wrap.poly, map.clip.poly)
S3method(identify, map)
useDynLib("maps", .registration = TRUE, .fixes="C_")

import("graphics")
importFrom("utils", "read.delim")
export(iso.expand,sov.expand,iso.alpha,area.map, identify.map, 
       map, map.axes, map.cities, map.scale, map.text, map.where, map.wrap, 
       match.map, smooth.map, .map.range,
       SpatialPolygons2map, SpatialLines2map, sf2map, map.wrap.poly, map.clip.poly)
S3method(identify, map)
useDynLib("maps", .registration = TRUE, .fixes="C_")

# maps
###R package for geographical maps

This is version 3 of the maps package. The main change with respect to v2 is a more recent world map. The old map legacy map is still available from the package mapdata.

##CHANGES

- The 'world' data base has been replaced by a much more recent data base, imported from the Natural Earth data project (the 1:50m world map, version 2.0, the latest version available in 2015)
This may have several implications for code that calls map().
  * As of version 3.1, the 'world' database no longer contains lakes and lake islands. These are now in a separate database called 'lakes'.
  * Country names have changed. This is inevitable, since e.g. post-1990 Europe is dramatically different. As a consequence, any call to map() that involves the argument "region=...", may be affected. For instance, map('world','USSR') is now obsolete, while map('world','Russia') works fine.
  * The new data base contains less small islands, but the choice may be a bit different.
  * Some remote islands that are officially part of other countries may cause an unexpected change in the scale of a map. This is not a new phenomenon (map('world','france') had this in the old set), but some new cases may occur now. As explained below, this will now occur *less often*. 
  * The naming of all polygons is as close as possible to the old world map. However, some inconsistencies have been remedied, e.g. on whether an island is called by its own name or as part of the country it belongs to. The choice is now mainly dependent on whether the entity has a seperate ISO code. The naming scheme unfortunately does not permit to represent all the intricacies. The data set iso3166 (see further) is added to provide more details.
  * Because of this segmentation by ISO code (and, often parallel, by admin-0 level), some countries now have much less islands. For instance, map(region="France") now only shows metropolitan France and one or two remote islands. To add all overseas territories and departments, you can use region=sov.expand("France"),  which creates a list of all countries under French sovereignty. 
  * The new 'world' database has a higher resolution than before and looks much smoother. In fact, for some applications it may now suffice rather than needing to import worldHires.

- Most functions now use perl-style regular expressions internally, rather than the default style. This gives more flexibility.

##ADDITIONS

- v3.1 includes support for 'SpatialPolygons' and 'SpatialLines' classes. Such objects can now either be passed to map() directly or first be converted to a map-type list with the new functions 'SpatialPolygons2map()' and 'SpatialLines2map'. These functions do not copy all the information present in the original data: only polygon names and co-ordinates are preserved (additional information like holes, plotting order etc. is ignored).

- A new databse 'lakes' has been added. The standard 'world' map no longer contains any lakes.

- A data frame iso3166 has been added which, for every country on the new world map, lists the official name, the ISO3166 2- and 3-letter codes, and also the sovereignty. Type '?iso3166' for details.

- A set of simple functions use this iso3166 table for creating a list of countries to map. So you can use the 2- or 3-letter ISO code (e.g. as country labels on a map), but also create a list based on the sovereignty. Type '?iso.expand' for details.

- v3.2 adds new options to map() for wrapping and exact boundary clipping.
- v3.4.3 adds initial support for 'sf' class maps.

##FIXES
- As of v3.2, maps::map() also works if the maps package is not attached. You can even do maps::map("madata::worldHires").
- As of v3.1, map(..., fill=TRUE) no longer applies thinning. This removes small artefacts, but plotting worldHires becomes rather slow, should you ever want to plot a full world map at such a high resolution.
- map.text(..., exact=TRUE) now behaves as documented. Previously, the "exact=TRUE" was not passed to the map drawing if add=FALSE. To get the old (non-documented) behaviour (plot map with exact=FALSE, write text for exact=TRUE) you should now use 2 commands: 
  * > map(..., exact=FALSE)
  * > map.text(..., exact=TRUE, add=TRUE)
- match.map now works correctly for regions containing ":". This was potentially broken due to locale-dependent behaviour of order().
- 'world2' now has clean boundaries when 'fill=TRUE'.

##FALL BACK TO LEGACY WORLD DATABASE
The old map data set has been moved to the 'mapdata' package, where it is called "worldLores". It can thus be called via map("mapdata::worldLores").

Note that the worldHires database from the mapdata package has identical map naming than the legacy world map. The only difference is in the resolution of the polylines. That high resolution version of the legacy map will remain unchanged.

Please inform the maintainer of any problem that requires a fallback to the legacy database!


## HIGH RESOLUTION AND OTHER MAPS

Natural Earth also supports a high-resolution (1:10m) world map. Rather than replacing 'worldHires' (in the mapdata package), this data set can be imported into maps e.g. by loading the "rnaturalearth" package.

Even maps in e.g. shapefile format can now easily be imported for use in 'map()'. For instance the package 'maptools' offers the function 'readShapePoly()', which reads a shapefile into a 'SpatialPolygonsDataFrame' that can be used as a map database (as of version 3.1).


##TO DO/DISCUSS:
- The old naming convention is largely maintained, but some choices are different. The changes mean that e.g. /region="France"/ now covers only metropolitan France, whithout (most of) the overseas departments and territories
- Some inconsistencies in the naming procedure remain. For instance, while most countries are named by their full name, "UK" and "USA" are shortened in the same way as in the old data base. For UK, this even required a hack in the mapping code to avoid adding Ukrain to the map.
- The iso3166 table may have to be adapted for the Natural Earth 1:10 database, some extra rows have already been inserted.

Notes on creating new map databases.

1) See the references:
Richard A. Becker, and Allan R. Wilks,
"Maps in S",
emph{AT\&T Bell Laboratories Statistics Research Report [93.2], 1993.}

Richard A. Becker, and Allan R. Wilks,
"Constructing a Geographical Database",
\emph{AT\&T Bell Laboratories Statistics Research Report [95.2], 1995.}

2) After the .line .gon and .n ascii files have been created or modified,
rerun ./getstats.sh to generate the appropriate .linestats and .gonstats
files.

3) Edit the Makefiles to add the appropriate names to the lists of
.G, .L and .N files

4) Create the ../data/<mapname>.r file consisting of the assignment
'<mapname>MapEnv <- "R_MAP_DATA_DIR"'.  This sets the name of the
environment variable, created at load time (see ../R/zzz.R), that
specifies the location of the binary data files for this collection
of maps. [The mapdata package uses a different environment variable.]

Ray Brownrigg
21 March 2009

NOTES ADDED:

5) Packages like 'maptools' have functionality to read e.g. shapefile data into SpatialPolygon objects. The new function SpatialPolygons2map() can be used as a first step to load such map data from various sources into the maps package. However, the output is similar to that of map(..., fill=TRUE). It does not include polyline information.

6) There is an R package that can ease thet transition from polygon based data to polylines;
https://githib.com:adeckmyn/mapMaker
This package was used to convert the current world and lakes databases from polygons to polylines. Some manual work remains to be done, depending on how clean and consistent the original data is.

Alex Deckmyn
19 Januari 2015

fix_exceptions <- function(patterns) {
  # we must distinguish uk from Ukraine...
  # very ad hoc, I know.
  patterns <- gsub("^UK$", "UK:", patterns, ignore.case=TRUE)
  patterns
}

fix_exceptions2 <- function(patterns=".") {
  stop("This function should not be called... yet.")
# FIX ME: will not work for exact=TRUE ("usa" should work)
  world.exceptions <- c("uk"="United Kingdom","usa"="United States of America")
  if (length(patterns)==1 && patterns==".") return(patterns)
  p1 <- vapply(strsplit(tolower(patterns),":"), function(x) x[1], FUN.VALUE="a")
  iexp <- which(p1 %in% names(world.exceptions))
  if (length(iexp)>0) {
    warning("The use of short names 'UK' and 'USA' is deprecated and will be removed in the future.")
    ilist <- unique(p1[iexp])
    for (ii in ilist) p2 <- gsub(ii, world.exceptions[ii], p2, ignore.case=TRUE)
    if (is.factor(patterns)) patterns <- as.character(patterns)
    patterns[iexp] <- p2
  }
  patterns
}

mapenvir <- function(database="world", warn.dep=FALSE) {
  dbname <- paste0(database, "MapEnv")
  # the following will allow us to find e.g. "world" even if
  # the maps package is not attached.
  if (length(grep("::", database)) == 0) {
    if (!exists(dbname)) dbname <- paste0("maps::",dbname)
  } else {
    database <- strsplit(database,"::")[[1]][2]
  }
  fbase <- paste0(Sys.getenv(eval(parse(text=dbname))), database)
  fbase
}

"mapgetg" <-
function(database = "world", gons, fill = FALSE, xlim = c(-1e30, 1e30),
	ylim = c(-1e30, 1e30))
{
	ngon <- length(gons)
	gnames <- names(gons)
	mapbase <- mapenvir(database, warn.dep=TRUE)
	z <- .C(C_map_getg,
		as.character(mapbase),
		gons = as.integer(gons),
		as.integer(ngon),
		sizes = integer(ngon),
		error = as.integer(0),
		as.double(c(xlim, ylim)),
		as.integer(fill))[c("gons", "sizes", "error")]
	gons <- z$gons
	sizes <- z$sizes
	if(z$error < 0)
		stop("error in reading polygon headers")
	z <- .C(C_map_getg,
		as.character(mapbase),
		as.integer(gons),
		as.integer(ngon),
		lines = integer(sum(sizes)),
		error = as.integer(1),
		as.double(c(xlim, ylim)),
		as.integer(fill))[c("lines", "error")]
	if(z$error < 0)
		stop("error in reading polyline numbers")
	lines <- z$lines
	ok <- sizes > 0
	list(number = lines, size = sizes[ok], name = gnames[ok])
}

"mapgetl" <-
function(database = "world", lines, xlim = c(-1e30, 1e30), ylim = c(-1e30,
	1e30), fill = FALSE)
{
	nline <- as.integer(length(lines))
	if(nline == 0)
		return(integer(0))
	mapbase <- mapenvir(database)
	z <- .C(C_map_getl,
		as.character(mapbase),
		linesize = as.integer(lines),
		error = as.integer(nline),
		as.integer(0),
		as.double(0),
		as.double(0),
		as.double(c(xlim, ylim)),
		as.integer(fill))[c("linesize", "error")]
	if(z$error < 0)
		return(integer(0))
	ok <- z$linesize != 0
	lines <- lines[ok]
	nline <- length(lines)
	if(nline == 0)
		return(integer(0))
	linesize <- z$linesize[ok]
	N <- sum(linesize) + nline - 1
	xy <- .C(C_map_getl,
		as.character(mapbase),
		as.integer(lines),
		as.integer(nline),
		as.integer(1),
		x = double(N),
		y = double(N),
		range = double(4),
		as.integer(fill))[c("x", "y", "range")]
	flip <- c(1, -1, 1, -1)
	box <- flip * pmax(flip * c(xlim, ylim), flip * xy$range)
	if(any(diff(box)[-2] <= 0)) {
		if(missing(xlim) || missing(ylim))
			stop("nothing to draw: data range and limits don't intersect"
				)
		else box <- c(xlim, ylim)
	}
	xy$range <- box
	xy
}

"mapname" <-
function(database = "world", patterns, exact = FALSE)
{
  mapbase <- mapenvir(database)
  # rewritten by Tom Minka
  fname <- paste(sep = "", mapbase, ".N")
  cnames <- read.delim(fname, as.is = TRUE, header = FALSE)
  nam <- as.character(cnames[[1]])
  if(exact) {
    i <- match(patterns, nam)
    if (any(is.na(i))) {
      if (!missing(patterns)) {
        pmiss <- patterns[is.na(i)]
        if (length(i)>0) warning(paste("Some patterns could not be exactly matched:\n   ",
                                       paste(pmiss,collapse=", "),"\n"))
      }
      i <- NULL
    }
  } else {
## QUICK FIX: there is a problem now for UK vs Ukrain...
    if (database=="world") patterns <- fix_exceptions(patterns)
    regexp <- paste("(^", patterns, ")", sep = "", collapse = "|")
# BUGFIX: perl regex is limited to about 30000 characters
# so this crashes if patterns includes the whole world map
    i <- grep(regexp, nam, ignore.case = TRUE,
              perl = (length(patterns) < 1000) & (nchar(regexp) < 30000))
  }
  if(length(i) == 0) return(NULL)
  r <- cnames[i, 2]
  names(r) <- nam[i]
  return(r)
}

"maptype" <-
function(database = "world")
{
  if(is.character(database)) {
	mapbase <- mapenvir(database)
        # minka: maptypes are now 1,2 instead of 0,1
	switch(.C(C_map_type,
		as.character(mapbase),
		integer(1))[[2]] + 2, "unknown", "spherical", "planar", "spherical")
  } else {
    # map object
    if (is.list(database)) {
      if (!is.null(database$maptype)) return(database$maptype)
#      if (!is.null(database$projection)) return("planar")
      return("spherical")
# you may also look at $projection ...
    } else "spherical"
  }
}

char.to.ascii <- function(s) {
  # returns the ascii code for a character (0 for an empty string)
  n = length(s)
  .C(C_char_to_ascii,
    as.integer(n), as.character(s), integer(n))[[2]]
}
is.regexp <- function(s) {
  # test: is.regexp(c("too", ".*dak", "kj[t]"))
  pattern = ".*[.?*^[].*"
  result = logical(length(s))
  result[grep(pattern, s)] = TRUE
  result
}
match.map <- function(database, regions, exact = FALSE, warn = TRUE) {
  invperm <- function(p) {
    # returns the inverse permutation, so that invperm(p)[p] = 1:n
    ip = p
    ip[p] = 1:length(p)
    ip
  }
  regions = tolower(regions)

  if(is.character(database)) {
    mapbase <- mapenvir(database)
    fname <- paste(sep = "", mapbase, ".N")
    x <- read.delim(fname, header = FALSE)
    nam <- as.character(x[[1]])

    # this is a quick-and-dirty fix for "uk" matching "ukrain"
    if (database=="world") regions <- fix_exceptions(regions)
  }
  else {
    nam <- database$names
  }
  nam = tolower(nam)

  if(!exact && any(is.regexp(regions))) {
    match.map.grep(nam, regions, warn)
  } else {
    # sort regions and table
    #AD: "order" may give unexpected results depending on locale!!!
    #so we temporarily move to ASCII style ordering
    lcc <- Sys.getlocale("LC_COLLATE")
    Sys.setlocale(category = "LC_COLLATE", locale = "C")
    ord.nam <- order(nam)
    nam <- nam[ord.nam]
    ord.regions <- order(regions)
    regions <- regions[ord.regions]
    Sys.setlocale(category = "LC_COLLATE", locale = lcc)

    result <- .C(C_map_match,
      as.integer(length(nam)), as.character(nam),
      as.integer(length(regions)), as.character(regions),
      result = integer(length(nam)), as.integer(exact))[["result"]]
    # 0 -> NA
    is.na(result[result == 0]) = TRUE
    # back to original order
    ord.regions[result[invperm(ord.nam)]]
    #result
  }
}
match.map.slow <- function(nam, regions, warn = FALSE) {
  result <- rep(NA, length = length(nam))
  # use hashing to prune the potential prefix matches
  # doesn't work for regexp
  let = min(nchar(regions))
  names(nam) = NULL
  hash = factor(char.to.ascii(substr(nam, let, let)))
  nam.bin = split(nam, hash)
  index.bin = split(1:length(nam), hash)
  for(i in 1:length(regions)) {
    pattern = regions[i]
    regexp <- paste("(^", pattern, ")", sep = "", collapse = "|")
    #r <- grep(regexp, nam)
    region.hash = as.character(char.to.ascii(substr(pattern, let, let)))
    r <- grep(regexp, nam.bin[[region.hash]], perl= TRUE )
    if(length(r) > 0) {
      r = index.bin[[region.hash]][r]
      result[r] <- i
    } else if(warn) warning(paste(pattern, "is not in the map"))
  }
  result
}
match.map.grep <- function(nam, regions, warn = FALSE) {
  result <- rep(NA, length = length(nam))
  for(i in 1:length(regions)) {
    pattern = regions[i]
    regexp <- paste("(^", pattern, ")", sep = "", collapse = "|")
    r <- grep(regexp, nam, perl=TRUE)
    if(length(r) > 0) {
      result[r] <- i
    } else if(warn) warning(paste(pattern, "is not in the map"))
  }
  result
}

# Some routines to create "map" objects from other formats

### 1. SP package
# transform a SpatialPolygons[DataFrame] into a list of polygons for map()
SpatialPolygons2map <- function(database, namefield=NULL){
  if (!inherits(database,"SpatialPolygons")) {
    stop("database must be a SpatialPolygons[DataFrame] object.")
  }
  region.names <- NULL
  if (inherits(database,"SpatialPolygonsDataFrame") & !is.null(namefield) ) {
    namcol <- lapply(namefield, function(x) which(tolower(names(database)) == tolower(x)))
    if (any(lapply(namcol, length) != 1)) {
      zz <- which(lapply(namcol, length) != 1)
      warning(paste0("database does not (uniquely) contain the field '",namefield[zz],"'."))
    } else {
      zz <- as.data.frame(lapply(database@data[unlist(namcol)], as.character), stringsAsFactors=FALSE)
      region.names <- vapply(1:dim(zz)[1], function(x) paste(zz[x,],collapse=":"),FUN.VALUE="a")
    }
  }
  if (is.null(region.names)) region.names <- unlist(lapply(database@polygons, function(x) x@ID))

  nregions <- length(region.names)

  # count the number of polygons in every "region"
  ngon <- vapply(1:nregions,
                 FUN=function(i) length(database@polygons[[i]]@Polygons),
                 FUN.VALUE=1)
  # if a region contains several polygons, an index is added to the name: "region:n"
  gon.names <- unlist(lapply(1:dim(database)[1], function(i) {
             if (ngon[i]==1) region.names[i]
             else paste(region.names[i],1:ngon[i],sep=":")}))

  # extract all polygon data to a list
  allpoly <- lapply(database@polygons,
                    function(x) lapply(x@Polygons, function(y) y@coords))
## allpoly is a list of lists of Nx2 matrices (not data frames)
## first flatten the list, then add NA to every row, then rbind and remove one NA
#  p1 <- do.call(c, allpoly)
#  p2 <- lapply(p1, function(x) rbind(c(NA,NA),x))
#  p3 <- do.call(rbind,p2)[-1,]
  mymap <- do.call(rbind, lapply(do.call(c,allpoly),
                                  function(x) rbind(c(NA,NA),x)))[-1,]
  result <- list(x = mymap[,1], y = mymap[,2], names = gon.names,
       range = c(range(mymap[,1], na.rm = TRUE),range(mymap[,2], na.rm = TRUE)))
  class(result) <- "map"
  result
}

# transform a SpatialLines[DataFrame] into a list of polylines for map()
SpatialLines2map <- function(database, namefield=NULL){
  if(!inherits(database,"SpatialLines")) stop("database must be a SpatialLines[DataFrame] object.")

  line.names <- NULL
  if (inherits(database,"SpatialLinesDataFrame") & !is.null(namefield) ) {
    namcol <- lapply(namefield, function(x) which(tolower(names(database)) == tolower(x)))
    if (any(lapply(namcol, length) != 1)) {
      zz <- which(lapply(namcol, length) != 1)
      warning(paste0("database does not (uniquely) contain the field '",namefield[zz],"'."))
    } else {
      zz <- as.data.frame(lapply(database@data[unlist(namcol)], as.character), stringsAsFactors=FALSE)
      line.names <- vapply(1:dim(zz)[1], function(x) paste(zz[x,],collapse=":"),FUN.VALUE="a")
    }
  }
  if (is.null(line.names)) line.names <- unlist(lapply(database@lines, function(x) x@ID))

  nlines <- length(line.names)

  # count the number of line segments in every "line"
  nseg <- vapply(1:nlines,
                 FUN=function(i) length(database@lines[[i]]@Lines),
                 FUN.VALUE=1)
  # if a line contains several sub-lines (segments), an index is added to the name: "line:n"
  line.names <- unlist(lapply(1:dim(database)[1], function(i) {
             if (nseg[i]==1) line.names[i]
             else paste(line.names[i],1:nseg[i],sep=":")}))

  # extract all polyline data to a list
  allpoly <- lapply(database@lines,
                    function(x) lapply(x@Lines, function(y) y@coords))
## allpoly is a list of lists of Nx2 matrices (not data frames)
## first flatten the list, then add NA to every row, then rbind and remove one NA
#  p1 <- do.call(c, allpoly)
#  p2 <- lapply(p1, function(x) rbind(c(NA,NA),x))
#  p3 <- do.call(rbind,p2)[-1,]
  mymap <- do.call(rbind, lapply(do.call(c,allpoly),
                                  function(x) rbind(c(NA,NA),x)))[-1,]
  result <- list(x = mymap[,1], y = mymap[,2], names = line.names,
       range = c(range(mymap[,1], na.rm = TRUE),range(mymap[,2], na.rm = TRUE)))
  class(result) <- "map"
  result
}

### 2. SF package (replacing sp)
sf2map <- function(database, namefield="name"){
  if (!inherits(database, "sf")) {
    stop("database must be a sf object.")
  }
  gname <- attr(database, "sf_column") # often ="geometry"
  tt = c("sfc_LINESTRING", "sfc_MULTILINESTRING", "sfc_POLYGON", "sfc_MULTIPOLYGON")
  if (!any(vapply(tt, FUN=function(x) inherits(database[[gname]], x), FUN.VALUE=TRUE))) {
    stop("sf2map currently only works for types ", paste(tt, collapse=" "))
  }
  nplines <- dim(database)[1]
  # in some cases (e.g. LINESTRING) various lines may have the same name, but we don't check
  if (!is.null(namefield) && namefield %in% names(database)) {
    # NOTE: the namefield may be a factor (e.g. tmap::World), so make sure to transform to vector
    line_names <- as.vector(database[[namefield]])
  } else {
    line_names <- 1:nplines
  }
  if (inherits(database[[gname]], "sfc_MULTILINESTRING") ||
      inherits(database[[gname]], "sfc_MULTIPOLYGON")) {
    nlines <- vapply(1:nplines,
                    FUN=function(r) sum(vapply(1:length(database[[gname]][[r]]),
                                               FUN=function(p) length(database[[gname]][[r]][[p]]),
                                               FUN.VALUE=1)),
                    FUN.VALUE=1)
    # if a multiline contains several lines, an index is added to the name: "line:n"
    line_names <- unlist(lapply(1:nplines, function(i) {
                    if (nlines[i]==1) line_names[i]
                    else paste(line_names[i], 1:nlines[i], sep=":")}))

    all_lines <- lapply(database[[gname]],
                      function(x) do.call(c, x))
  } else {
    all_lines <- database[[gname]]
  }
  # extract all line data to a list

## all_lines is a list of lists of Nx2 matrices (not data frames)
## first flatten the list, then add NA to every row, then rbind and remove one NA
#  p1 <- do.call(c, all_lines)
#  p2 <- lapply(p1, function(x) rbind(c(NA,NA),x))
#  p3 <- do.call(rbind,p2)[-1,]
  mymap <- do.call(rbind, lapply(do.call(c,all_lines),
                                  function(x) rbind(c(NA,NA),x)))[-1,]
  result <- list(x = mymap[,1], y = mymap[,2], names = line_names,
                 range = c(range(mymap[,1], na.rm = TRUE),
                           range(mymap[,2], na.rm = TRUE)))
  class(result) <- "map"
  result
}


###########################################

.onLoad <- function(lib, pkg) {
  if (Sys.getenv("R_MAP_DATA_DIR") == "") {
    Sys.setenv("R_MAP_DATA_DIR" = paste(lib, pkg, "mapdata/", sep="/"))
  }
  ## for now, we keep the option of having a different world map
  if (Sys.getenv("R_MAP_DATA_DIR_WORLD") == "") {
    Sys.setenv("R_MAP_DATA_DIR_WORLD" = Sys.getenv("R_MAP_DATA_DIR"))
  }
  library.dynam("maps", pkg, lib)
}

"mapthin" <-
function(xy, delta, symmetric = TRUE)
{
  x <- xy$x
  y <- xy$y
  xy <- .C(C_map_thin,
    x = as.double(x),
    y = as.double(y),
    n = as.integer(length(x)),
    as.double(delta),
    as.integer(symmetric),
    NAOK = TRUE)[c("x", "y", "n")]
  length(xy$x) <- xy$n
  length(xy$y) <- xy$n
  xy[c("x", "y")]
}

# add axes to a map
"map.axes" <-
function(...)
{
  axis(1, ...)
  axis(2, ...)
  box(...)
  invisible()
}

"map.cities" <-
function (x = world.cities, country = "", label = NULL, minpop = 0,
  maxpop = Inf, capitals = 0, cex = par("cex"), projection = FALSE,
  parameters = NULL, orientation = NULL, pch = 1, ...)
{
  if (missing(x) && !exists("world.cities")) {
    world.cities <- maps::world.cities
  }
  usr <- par("usr")
  if (!missing(projection) && projection != FALSE) {
    if (requireNamespace("mapproj", quietly = TRUE)) {
      if (is.character(projection)) {
        projx <- mapproj::mapproject(x$long, x$lat, projection = projection,
          parameters = parameters, orientation = orientation)
      } else {
        if (nchar(mapproj::.Last.projection()$projection) > 0) {
          projx <- mapproj::mapproject(x$long, x$lat)
        } else stop("No projection defined\n")
      }
      x$long <- projx$x
      x$lat <- projx$y
    } else stop("mapproj package not available\n")
  } else {
    if (usr[2] > (180 + 0.04*(usr[2] - usr[1])))
      x$long[x$long < 0] <- 360 + x$long[x$long < 0]
  }
  selection <- x$long >= usr[1] & x$long <= usr[2] & x$lat >= usr[3] &
    x$lat <= usr[4] & (x$pop >= minpop & x$pop <= maxpop) & ((capitals == 0) |
    (x$capital >= 1))
  if (country != "")
    selection <- selection & x$country.etc == country
  selection0 <- selection & (x$capital == 0) & (capitals == 0)
  selection01 <- selection & (x$capital <= 1) & (capitals <= 1)
  selection1 <- selection & (x$capital == 1) & (capitals == 1)
  selection2 <- selection & (x$capital == 2) & (capitals == 2)
  selection3 <- selection & (x$capital == 3) & (capitals == 3)
  if (is.null(label))
    label <- sum(selection) < 20
  cxy <- par("cxy")
  if (sum(selection01) > 0)
    points(x$long[selection01], x$lat[selection01], pch = pch,
      cex = cex * 0.6, ...)
  if (sum(selection0) > 0)
    if (label)
      text(x$long[selection0], x$lat[selection0] + cxy[2] * cex * 0.7,
        paste(" ", x$name[selection0], sep = ""), cex = cex * 0.7, ...)
  if (sum(selection1) > 0) {
    points(x$long[selection1], x$lat[selection1], pch = pch, cex = cex, ...)
    if (label) {
      text(x$long[selection1], x$lat[selection1] + cxy[2] * cex,
        paste(" ", x$name[selection1], sep = ""), cex = cex * 1.2, ...)
    }
  }
  if (sum(selection2) > 0) {
    points(x$long[selection2], x$lat[selection2], pch = pch, cex = cex, ...)
    if (label) {
      text(x$long[selection2], x$lat[selection2] + cxy[2] * cex * 1.1,
        paste(" ", x$name[selection2], sep = ""), cex = cex * 1.1, ...)
    }
  }
  if (sum(selection3) > 0) {
    points(x$long[selection3], x$lat[selection3], pch = pch, cex = cex, ...)
    if (label) {
      text(x$long[selection3], x$lat[selection3] + cxy[2] * cex * 0.9,
        paste(" ", x$name[selection3], sep = ""), cex = cex * 0.9, ...)
    }
  }
  invisible()
}

# draw a scale bar on a map
"map.scale" <-
function (x, y, relwidth = 0.15, metric = TRUE, ratio = TRUE, ...)
{
  # old version
  # format.pretty <- function(x) {
  #   as.character(pretty(x * c(0.99, 1.01), n = 2)[2])
  # }
  # minka: new version
  format.pretty <- function(x, digits = 2) {
  x = signif(x, 2)
  prettyNum(formatC(x, format = "fg", digits = digits), big.mark = ",")
  }
  usr <- par("usr")
  if (missing(y))
  y <- (9 * usr[3] + usr[4])/10
  if (abs(y) >= 90)
  warning("location of scale out of this world!")
  if (missing(x))
  #x <- (0.9 - relwidth) * usr[2] + (0.1 + relwidth) * usr[1]
  x <- (9 * usr[1] + usr[2])/10
  cosy <- cos((2 * pi * y)/360)
  perdeg <- (2 * pi * (6356.78 + 21.38 * cosy) * cosy)/360
  scale <- (perdeg * 100000)/(2.54 * (par("pin")/diff(par("usr"))[-2])[1])
  if (metric)
  unit <- "km"
  else {
  perdeg <- perdeg * 0.6213712
  unit <- "mi"
  }
  len <- perdeg * relwidth * (usr[2] - usr[1])
  ats <- pretty(c(0, len), n = 2)
  nats <- length(ats)
  labs <- as.character(ats)
  labs[nats] <- paste(labs[nats], unit)
  linexy <- matrix(NA, ncol = 2, nrow = 3 * nats)
  colnames(linexy) <- c("x", "y")
  cxy <- par("cxy")
  dy <- cxy[2] * par("tcl")
  dx <- ats[nats]/perdeg/(nats - 1)
  linexy[1, ] <- c(x, y)
  linexy[2, ] <- c(x, y + dy)
  for (i in 1:(nats - 1)) {
  linexy[3 * i, ] <- c(x + (i - 1) * dx, y)
  linexy[3 * i + 1, ] <- c(x + i * dx, y)
  linexy[3 * i + 2, ] <- c(x + i * dx, y + dy)
  }
  lines(linexy)
  # minka: this is broken
  text(x + ats/perdeg, y + dy - 0.5 * cxy[2], labs, adj = c(0.4, 0.5), ...)
  # minka: added ratio option
  if(ratio)
  text(x, y + 0.5 * cxy[2],
     paste("scale approx 1:", format.pretty(scale), sep = ""),
     adj = 0, ...)
  invisible(scale)
}

map.wrap <- function(p, xlim=NULL) {
  # new version Alex Deckmyn
  # faster, a bit more robust (check data range), bugs fixed, xlim option added
  # insert NAs to break lines that wrap around the globe.
  # does not work properly with polygons.
  # p is list of x and y vectors
  # xc is the central value of the longitude co-ordinate: usually 0, but world2 has [0,360], so 180
  if (is.null(xlim)) {
    xc <- 0
    xr <- diff(range(p$x, na.rm=TRUE))
  } else {
    xc <- mean(xlim)
    xr <- diff(xlim)
  }
  dx <- abs(diff(p$x - xc))
  dax <- abs(diff(abs(p$x - xc)))
  j <- which(dx/dax > 50 & dx > (xr * 0.8) )
  if (length(j)==0) return(p)
  j <- c(j, length(p$x))
  ind <- seq_along(p$x)

  index <- c(ind[1:j[1]],
             unlist(lapply(1:(length(j)-1), function(k) c(NA, ind[(j[k]+1):j[(k + 1)]]) )) )
# notice that p$name is not returned!
# so if wrapping is applied, you loose polygon names
# but they would be wrong, anyway...
  list(x = p$x[index], y = p$y[index])
}

# a temporary function to switch to the old legacy world map (1990)
world.legacy <- function(wl=FALSE){
  if(wl) Sys.setenv("R_MAP_DATA_DIR_WORLD" =
                  paste(Sys.getenv("R_MAP_DATA_DIR"),"legacy_", sep="/"))
  else Sys.setenv("R_MAP_DATA_DIR_WORLD" = Sys.getenv("R_MAP_DATA_DIR"))
  .Deprecated(new='map("mapdata::worldLores",...)',
              package="maps")
}

# R data structure for maps is list(x, y, names)
# names is character vector naming the polygons
# (contiguous groups of non-NA coordinates)

# returns the jth contiguous section of non-NA values in vector x
# should consecutive NA's count as one?
subgroup <- function(x, i) {
  n <- length(x)
  breaks <- which(is.na(x))
  if (length(breaks) == 0) {
    starts <- 1
    ends <- n
  } else {
    starts <- c(1, breaks + 1)
    ends <- c(breaks - 1, n)
  }
## AD: EXTRA: allow negative values in i reverse direction
  pl <- function(j) if(j>0) x[starts[j]:ends[j]] else x[ends[abs(j)]:starts[abs(j)]]
  as.numeric(unlist(lapply(i,function(j) c(NA,pl(j))))[-1])
}

sub.polygon <- function(p, i) {
  lapply(p[c("x", "y")], function(x) subgroup(x, i))
}

# FUTURE:
#sub.gondata <- function(p, i) {
#  x <- p$gondata
#  n <- length(x)
#  breaks <- which(is.na(x))
#  if (length(breaks) == 0) {
#    starts <- 1
#    ends <- n
#  } else {
#    starts <- c(1, breaks + 1)
#    ends <- c(breaks - 1, n)
#  }
#  lengths <- starts - ends + 1
#  i2 <- unlist(lapply(i,function(j) c(NA,x[starts[j]:ends[j]])))[-1]
#  xy <- sub.polygon(p, i2)
#}

# returns a sub-map of the named map corresponding to the given regions
# regions is a vector of regular expressions to match to the names in the map
# regions outside of xlim, ylim may be omitted
map.poly <- function(database, regions = ".", exact = FALSE,
                     xlim = NULL, ylim = NULL, boundary = TRUE,
		     interior = TRUE, fill = FALSE, as.polygon = FALSE, namefield="name") {
  if (!is.character(database)) {
    if (!as.polygon) stop("map objects require as.polygon=TRUE")
    if (inherits(database, "sf")) {
      the.map <- sf2map(database, namefield=namefield)
    } else if (inherits(database,"SpatialPolygons")) {
      the.map <- SpatialPolygons2map(database, namefield=namefield)
    } else if (inherits(database,"SpatialLines")) {
      the.map <- SpatialLines2map(database, namefield=namefield)
    } else if (is.list(database) &&
               all(c("x", "y", "names") %in%  names(database))) {
      the.map <- database
    } else {
      stop("database type not supported.")
    }
    if (identical(regions,".")) {
      # speed up the common case
      nam = the.map$names
      coord <- the.map[c("x", "y")]
    } else {
      # same as mapname()
      if (exact) {
        i = match(regions, the.map$names)
        if (any(is.na(i))) i = NULL
      } else {
## AD TODO: solve the problem for UK vs Ukrain...
## But here you have a simple set of named polygons, not a map database.
## If it comes from anything but "world", we could break something else.
## We just leave it for now.
        regexp <- paste("(^", regions, ")", sep = "", collapse = "|")
## FIXME: find a better solution
# perl regex crashes on >30000 characters, e.g. whole world database
        i <- grep(regexp, the.map$names, ignore.case = TRUE,
                  perl = (length(regions) < 1000) & (nchar(regexp) < 30000) )
      }
      if (length(i) == 0) stop("no recognized region names")
      nam <- the.map$names[i]
      coord <- sub.polygon(the.map, i)
    }
    coord$range <- c(range(coord$x, na.rm = TRUE), range(coord$y, na.rm = TRUE))
  } else {
    # turn the polygon numbers into a list of polyline numbers
    gon <- mapname(database, regions, exact)
    n <- length(gon)
    if (n == 0) stop("no recognized region names")
    if (is.null(xlim)) xlim <- c(-1e+30, 1e+30)
    if (is.null(ylim)) ylim <- c(-1e+30, 1e+30)
    # turn the polygon numbers into a list of polyline numbers
    line <- mapgetg(database, gon, as.polygon, xlim, ylim)
    if (length(line$number) == 0)
            stop("nothing to draw: all regions out of bounds")
    # turn the polyline numbers into x and y coordinates
    if (as.polygon) {
      coord <- mapgetl(database, line$number, xlim, ylim, fill)
      # assemble lines into polygons
      gonsize <- line$size
      keep <- rep(TRUE, length(gonsize))
      coord[c("x", "y")] <- makepoly(coord, gonsize, keep)
    }
    else {
      l <- abs(line$number)
      if (boundary && interior) l <- unique(l)
      else if (boundary) l <- l[!match(l, l[duplicated(l)], FALSE)]
      else l <- l[duplicated(l)]
      coord <- mapgetl(database, l, xlim, ylim, fill)
      if (length(coord) == 0)
              stop("all data out of bounds")
    }
    nam <- line$name
  }
  list(x = coord$x, y = coord$y, range = coord$range, names = nam)
}

.map.range <-
local({
   val <- NULL
   function(new) if(!missing(new)) val <<- new else val
})

map <-
function(database = "world", regions = ".", exact = FALSE,
	 boundary = TRUE, interior = TRUE, projection = "",
	 parameters = NULL, orientation = NULL, fill = FALSE,
	 col = 1, plot = TRUE, add = FALSE, namesonly = FALSE,
         xlim = NULL, ylim = NULL, wrap = FALSE,
         resolution = if (plot) 1 else 0, type = "l", bg = par("bg"),
         mar = c(4.1, 4.1, par("mar")[3], 0.1), myborder = 0.01,
         namefield="name", lforce = "n", ...)
{
  # parameter checks
  if (resolution>0 && !plot)
    stop("must have plot=TRUE if resolution is given")
  if (!fill && !boundary && !interior)
    stop("one of boundary and interior must be TRUE")
  doproj <- !missing(projection) || !missing(parameters) || !missing(
          orientation)
  if (doproj && !requireNamespace("mapproj", quietly=TRUE)) {
    stop("Please install the package 'mapproj' for projections.")
  }
  coordtype <- maptype(database)
  if (coordtype == "unknown")
     stop("missing database or unknown coordinate type")
  if (doproj && coordtype != "spherical")
    stop(paste(database, "database is not spherical; projections not allowed"))
  if (length(wrap)>=2 && !doproj && wrap[2] - wrap[1] != 360)
    stop("The specified longitudes for wrapping are inconsistent, they should be 360 apart.")
  # turn the region names into x and y coordinates
  if (is.character(database)) as.polygon = fill
  else as.polygon <- TRUE
  # if we are going to wrap around anything else than [-180, 180], the simplest way
  # to get all necessary polylines/polygons is to set xlim=NULL *temporarily*
  # (alternatively, the C code must shift longitudes by +/- 360 to fit in boundaries)
  xlim_tmp <- if (length(wrap)>=2) NULL else xlim
  coord <- map.poly(database, regions, exact, xlim_tmp, ylim,
                    boundary, interior, fill, as.polygon, namefield=namefield)
  if (is.na(coord$x[1])) stop("first coordinate is NA. Bad map data?")
  if (length(wrap)>=2) {
    antarctica <- if (length(wrap) == 2) -89.9 else wrap[3]
    coord <- map.wrap.poly(coord, xlim=wrap[1:2], poly=fill, antarctica=antarctica)
  }
  # we can enforce xlim & ylim exactly
  # this changes the output
  if (lforce=="e") {
    coord <- map.clip.poly(coord, xlim, ylim, poly=fill)
  }
  if (plot) {
    .map.range(coord$range)
  }

  if (doproj) {
    nam <- coord$names
    coord <- mapproj::mapproject(coord, projection = projection,
			parameters = parameters, orientation = orientation)
    coord$projection = projection
    coord$parameters = parameters
    coord$orientation = orientation
    coord$names <- nam
    if (!is.null(xlim) && !is.null(ylim) && lforce %in% c("s","l")) {
      prange <- mapproj::mapproject(x=rep(xlim,2), y=rep(ylim, each=2))
      if (lforce=="s") {
        xlim <- c(max(prange$x[c(1,3)]),min(prange$x[c(2,4)]))
        ylim <- c(max(prange$y[c(1,2)]),min(prange$y[c(3,4)]))
      } else {
        xlim <- c(min(prange$x[c(1,3)]),max(prange$x[c(2,4)]))
        ylim <- c(min(prange$y[c(1,2)]),max(prange$y[c(3,4)]))
      }
    }
    if (plot && coord$error)
      if (all(is.na(coord$x)))
        stop("projection failed for all data")
      else warning("projection failed for some data")
  }
  # AD: we do wrapping first: slightly better than when run after the thinning
  #     also now the output data is also wrapped if plot=FALSE
  if (length(wrap)==1 && wrap) coord <- map.wrap(coord)
  # do the plotting, if requested
  if (plot) {
    # for new plots, set up the coordinate system;
    # if a projection was done, set the aspect ratio
    # to 1, else set it so that a long-lat square appears
    # square in the middle of the plot
    if (!add) {
      opar = par(bg = bg)
      if (!par("new")) plot.new()
      # xlim, ylim apply before projection unless we have pforce=TRUE
      if (is.null(xlim) || (doproj && !(lforce %in% c("s","l")))) xrange <- range(coord$x, na.rm = TRUE)
      else xrange <- xlim
      if (is.null(ylim) || (doproj && !(lforce %in% c("s","l")))) yrange <- range(coord$y, na.rm = TRUE)
      else yrange <- ylim
      if (coordtype != "spherical" || doproj) {
	aspect <- c(1, 1)
      } else
        aspect <- c(cos((mean(yrange) * pi)/180), 1)
      d <- c(diff(xrange), diff(yrange)) * (1 + 2 * myborder) * aspect
      if (coordtype != "spherical" || doproj) {
        plot.window(xrange, yrange, asp = 1/aspect[1])
      } else {
        # must have par(xpd = FALSE) for limits to have an effect ??!
	par(mar = mar)	# set mai if user-defined mar
	p <- par("fin") -
	  as.vector(matrix(c(0, 1, 1, 0, 0, 1, 1, 0), nrow = 2) %*% par("mai"))
	par(pin = p)
        p <- par("pin")
        p <- d * min(p/d)
        par(pin = p)
        d <- d * myborder + ((p/min(p/d) - d)/2)/aspect
        usr <- c(xrange, yrange) + rep(c(-1, 1), 2) * rep(d, c(2, 2))
        par(usr = usr)
      }
      on.exit(par(opar))
    }
    if (type != "n") {
      # thinning only works correctly if you have polylines from a database
      if (!as.polygon && resolution != 0) {
        pin <- par("pin")
        usr <- par("usr")
        resolution <- resolution * min(diff(usr)[-2]/pin/100)
        coord[c("x", "y")] <- mapthin(coord, resolution)
      }
      if (fill) polygon(coord, col = col, ...)
      else lines(coord, col = col, type = type, ...)
    }
  }
  # return value is names or coords, but not both
  class(coord) = "map"
  value <- if (namesonly) coord$names else coord
  if (plot) invisible(value)
  else value
}

"makepoly" <-
function(xy, gonsize, keep)
{
  # remove NAs and duplicate points so that a set of polylines becomes a set
  # of polygons.
  # xy is a set of polylines, separated by NAs.
  # gonsize is a vector, giving the number of lines to put into each polygon
  # note that a polyline may consist of a single point
  x <- xy$x
  y <- xy$y
  n <- length(x)
  gonsize <- gonsize[ - length(gonsize)]
  discard <- seq(length(x))[is.na(x)]
  if (length(discard) > 0) {
    # locations of (possible) duplicate points
    dups = c(discard - 1, n)
    # only polylines with > 1 point have duplicates
    i = which(diff(c(0, dups)) > 2);
    discard <- c(discard, dups[i]);
  }
  # first part of discard is the NAs, second part is duplicates
  # gonsize tells us which NAs to preserve
  if (length(gonsize) > 0)
    discard <- discard[ - cumsum(gonsize)]
  if (length(discard) > 0) {
    x <- x[ - discard]
    y <- y[ - discard]
  }
  keep <- rep(keep, diff(c(0, seq(length(x))[is.na(x)], length(x))))
  closed.polygon(list(x = x[keep], y = y[keep]))
}
closed.polygon <- function(p) {
  # p is a set of polylines, separated by NAs
  # for each one, the first point is copied to the end, giving a closed polygon
  x = p$x
  y = p$y
  n = length(x)
  breaks <- seq(length(x))[is.na(x)]
  starts <- c(1, breaks + 1)
  ends <- c(breaks - 1, n)
  x[ends + 1] = x[starts]
  y[ends + 1] = y[starts]
  x = insert(x, breaks + 1)
  y = insert(y, breaks + 1)
  list(x = x, y = y)
}
insert <- function(x, i, v = NA) {
  # insert v into an array x, at positions i
  # e.g. insert(1:7, c(2, 5, 8))
  n = length(x)
  new.n = n + length(i)
  m = logical(new.n)
  i.new = i - 1 + seq(length(i))
  m[i.new] = TRUE
  x = x[(1:new.n) - cumsum(m)]
  x[i.new] = v
  x
}

map.where <- function(database = "world", x, y, ...)
{
  if(missing(y)) {
    if(is.matrix(x)) { y <- x[, 2]; x <- x[, 1] }
    else if(is.list(x) && !is.null(x$y)) { y <- x$y; x <- x$x }
    else { y <- x[[2]]; x <- x[[1]] }
  }
  if(is.character(database)) {
    mapbase <- mapenvir(database)
    gon <- .C(C_map_where,
       as.character(mapbase),
       as.double(x),
       as.double(y),
       as.integer(length(x)),
       integer(length(x)),
       NAOK=TRUE)[[5]]
    # this must be database, not mapbase
    nam <- mapname(database, ".")
    gon[gon == 0] = NA
    # names(nam)[gon] #AD: this only works if database.N is perfectly ordered
    names(nam)[match(gon,nam)]
  }
  else {
    if (inherits(database,"SpatialPolygons")) {
      database <- SpatialPolygons2map(database, ...)
    }
    if(num.polygons(database) != length(database$names))
      stop("map object must have polygons (fill=TRUE)")
    n <- length(database$x)
    result <- .C(C_map_in_polygon,
       as.double(database$x), as.double(database$y), as.integer(n),
       as.double(x), as.double(y), as.integer(length(x)),
       integer(length(x)), NAOK = TRUE)[[7]]
    result[result == 0] = NA
    database$names[result]
  }
}
as.matrix.polygon <- function(x, ...) {
  p = x
  if(is.null(p)) return(p)
  if(is.list(p) && !is.data.frame(p)) p <- cbind(p$x, p$y)
  p
}
in.one.polygon <- function(p, x) {
  # returns a logical vector, whose length is nrow(x)
  if(is.null(p)) return(NA)
  p <- as.matrix.polygon(p)
  if(is.list(x) && !is.data.frame(x)) x <- cbind(x$x, x$y)
  if(is.vector(x)) dim(x) <- c(1, 2)
  # p and x are matrices
  .C(C_map_in_one_polygon,
     as.double(p[, 1]), as.double(p[, 2]), as.integer(nrow(p)),
     as.double(x[, 1]), as.double(x[, 2]), as.integer(nrow(x)),
     logical(nrow(x)), as.integer(TRUE))[[7]]
}
in.polygon <- function(p, x) {
  # returns a logical vector, whose length is nrow(x)
  if(is.null(p)) return(NA)
  p <- as.matrix.polygon(p)
  if(is.list(x) && !is.data.frame(x)) x <- cbind(x$x, x$y)
  if(is.vector(x)) dim(x) <- c(1, 2)
  # p and x are matrices
  .C(C_map_in_polygon,
     as.double(p[, 1]), as.double(p[, 2]), as.integer(nrow(p)),
     as.double(x[, 1]), as.double(x[, 2]), as.integer(nrow(x)),
     logical(nrow(x)), NAOK = TRUE)[[7]] > 0
}

# polygon is not assumed closed
area.polygon <- function(p) {
  if(is.null(p)) return(NA)
  p <- as.matrix.polygon(p)
  n <- nrow(p)
  x1 <- p[, 1]
  i2 <- c(n, 1:(n - 1))
  x2 <- p[i2, 1]
  y1 <- p[, 2]
  y2 <- p[i2, 2]
  0.5*abs(sum(x1*y2 - x2*y1))
}

centroid.polygon <- function(p) {
  if(is.null(p)) return(c(NA, NA))
  p <- as.matrix.polygon(p)
  n <- nrow(p)
  x1 <- p[, 1]
  i2 <- c(n, 1:(n - 1))
  x2 <- p[i2, 1]
  y1 <- p[, 2]
  y2 <- p[i2, 2]
  a <- x1*y2 - x2*y1
  s <- sum(a)*3
  if(s == 0) c(mean(x1), mean(y1))
  else c(sum((x1 + x2)*a)/s, sum((y1 + y2)*a)/s)
}

# applies fun to all sub-polygons of p
apply.polygon <- function(p, fun, names. = NULL) {
  if(is.null(p)) return(p)
  if(is.null(names.) && !is.null(p$names)) names. = p$names
  p = as.matrix.polygon(p)
  n <- nrow(p)
  breaks <- (1:n)[is.na(p[, 1])]
  starts <- c(1, breaks + 1)
  ends <- c(breaks - 1, n)
  m <- length(starts)
  result <- list()
  for(i in 1:m) {
    this.p = if(ends[i] >= starts[i]) p[starts[i]:ends[i], ] else NULL
    result[[i]] <- fun(this.p)
  }
  names(result) = names.
  result
}

num.polygons <- function(p) {
  if(is.list(p)) 1 + sum(is.na(p$x))
  else 1 + sum(is.na(p[, 1]))
}

# range.polygon <- function(..., na.rm = FALSE) {
#   p <- as.list.polygon(...)
#   lapply(p[c("x", "y")], range, na.rm = na.rm)
# }

map.text <- function(database, regions = ".", exact = FALSE, labels,
    cex = 0.75, add = FALSE, move = FALSE, ...) {
  if(!add) map(database=database, regions=regions, exact=exact, ...)
  # get polygons
  cc = match.call(expand.dots=TRUE)
  cc[[1]] = as.name("map")
  cc$fill = TRUE
  cc$plot = FALSE
  cc$regions = regions
  cc$exact = exact
  cc$move = cc$add = cc$cex = cc$labels = NULL
  cc$resolution = 0
  m = eval(cc)
  if(missing(labels)) {
    labels = gsub(".*,", "", m$names)
  }
  if(num.polygons(m) != length(labels))
    stop("map object must have polygons (fill=TRUE) and equal number of labels")
  x = apply.polygon(m, centroid.polygon)
  # convert m into a matrix
  x <- t(array(unlist(x), c(2, length(x))))
  if(move) {
# AD: this option should probably be removed, as the code is not available
    # require(mining)
    move.collisions2 <- get("move.collisions2")	# to prevent check NOTE
    w = strwidth(labels, units = "inches", cex = cex)
    h = strheight(labels, units = "inches", cex = cex)
    x = move.collisions2(x[, 1], x[, 2], w, h)
  }
  # want to omit map-specific options here (like "exact")
  text(x, labels, cex = cex, ...)
  invisible(m)
}

identify.map <- function(x, n = 1, index = FALSE, ...) {
  # identify polygons in a map
  # must click near the center of the polygon
  m = x
  if(!is.list(m)) stop("must provide a map object")
  if(num.polygons(m) != length(m$names))
    stop("map object must have polygons (fill=TRUE)")
  x = apply.polygon(m, centroid.polygon)
  x <- t(array(unlist(x), c(2, length(x))))
  i = identify(x[, 1], x[, 2], labels = m$names, n = n, ...)
  if(index) i else m$names[i]
}

area.map <- function(m, regions = ".", sqmi=TRUE, ...) {
  # returns the areas of given regions,
  # combining the areas of all regions which match.
  if(!is.list(m)) stop("must provide a map object")
  if(num.polygons(m) != length(m$names))
    stop("map object must have polygons (fill=TRUE)")
  proj = m$projection
  m = map.poly(m,regions,as.polygon=TRUE,...)
  area = unlist(apply.polygon(m, area.polygon))
  merge <- regions[match.map(m, regions, ...)]
  names(merge) <- m$names
  merge = factor(merge, levels = regions)
  area = drop(indicators.factor(merge) %*% area)
  areaSqMiles <- function(proj) {
    # returns a factor f such that f*area.map() is in square miles.
    if(is.null(proj)) proj = "no projection"
    if(proj %in% c("mollweide","azequalarea","aitoff")) 2*15732635
    else if(proj %in% c("sinusoidal","bonne","cylequalarea","albers")) 15732635
    else if(proj == "sp_albers") 15745196
    else {
      warning(paste("sq.mile correction unavailable for",proj))
      1
    }
  }
  if(sqmi) area*areaSqMiles(proj) else area
}
indicators.factor <- function(y) {
  # convert a factor into a matrix of indicators
  # result is level by case
  # works if y contains NAs
  r <- array(0, c(length(levels(y)), length(y)), list(levels(y), names(y)))
  for(lev in levels(y)) r[lev, y == lev] <- 1
  r
}

### translate ISO 3166-1 alpha-2 codes to country names
### default result is a character vector of same length, which may contain regular expressions
iso.expand <- function(a,regex=TRUE){
  if (!exists("iso3166")) iso3166 <- maps::iso3166
  AA <- toupper(a)
  if (all(nchar(a)==2)) codes <- iso3166$a2
  else if (all(nchar(a)==3)) codes <- iso3166$a3
  else stop("All codes must be equal length, 2 or 3 characters.")
  nn <- lapply(AA,function(x) iso3166$mapname[which(codes == x)])
  if (regex) {
    nn2 <- lapply(nn,function(x) if (length(x)>1) paste("(^",x,")",sep="",collapse="|") else x)
    unlist(nn2)
  } else unlist(nn)
}

### list all countries that fall under sovereignty of 'sov'
sov.expand <- function(sov,regex=TRUE){
  if (!exists("iso3166")) iso3166 <- maps::iso3166
  sov <- tolower(sov)
  sel <- tolower(iso3166$sovereignty)
  nn <- lapply(sov,function(x) iso3166$mapname[which(sel == x)])
  if (regex) {
    nn2 <- lapply(nn,function(x) if (length(x)>1) paste("(^",x,")",sep="",collapse="|") else x)
    unlist(nn2)
  } else unlist(nn)
}

### the inverse:
### The subtlety lies in dealing with special cases like "China:Hong Kong" which has "HK", not "CN"
### But I think it's OK if "China" returns "CN", not "CN"+"HK"+"MC".
### Similar for Norway/Svalbard, Finland/Aland
### But of course, "China:Hong Kong" must return "HK" etc.

### we does this with a reverse grep
### downsides: the names in x must be complete
iso.alpha <- function(x,n=2){
  if (!exists("iso3166")) iso3166 <- maps::iso3166

## part 1: reverse fit will find all special cases, but names in vector x must be complete
  nam1 <- lapply(seq_along(iso3166$mapname),
                    function(nn) {regex <- paste("(^",iso3166$mapname[nn],")",sep="")
                                  ttt <- grep(regex, x, perl=TRUE, ignore.case=TRUE);
                                  if (length(ttt)>0) cbind(nn,ttt) else NULL})
  fli <- do.call(rbind,nam1)
  sel <- fli[match(seq_along(x),fli[,2]),1]

## part 2:
## try for partial fit. if it gives a single result, use it.
  if (any(is.na(sel))) {
    sel2 <- which(is.na(sel))
    nam2 <- unlist(lapply(x[sel2],
              function(nn) {regx <- paste("(^",nn,")",sep="") ;
                            ttt <- grep(regx, iso3166$mapname, perl=TRUE, ignore.case=TRUE);
                            if (length(ttt)==1) ttt else NA}))
    sel[sel2] <- nam2
  }

  if (n==2) iso3166$a2[sel]
  else if (n==3) iso3166$a3[sel]
  else stop("n must be 2 or 3.")
}

map.wrap.poly <- function(data, xlim, poly = FALSE, antarctica = -89.5) {
  nseg <- sum(is.na(data$x))+1
  len_in <- length(data$x)
  # usually, the result will be less then twice as long,
  # but every wrapped boundary (poly=TRUE) adds 10 interpolation points
  # so for a small data set, this can be more than x2
  len_out <- 2*len_in + 100
  wrap <- .C(C_map_wrap_poly,
               xin=data$x, yin=data$y, nin=as.integer(len_in),
               xout=numeric(len_out), yout=numeric(len_out), 
               nout=as.integer(len_out),
               xmin=as.numeric(xlim[1]), xmax=as.numeric(xlim[2]),
               poly=as.integer(poly), npoly=integer(nseg),
               antarctica=as.double(antarctica),
               NAOK=TRUE)

  xlen <- wrap$nout
  data$x <- wrap$xout[1:xlen]
  data$y <- wrap$yout[1:xlen]
  if (!is.null(data$names) && poly) {
    data$names <- rep(data$names, times=wrap$npoly)
  }
  data$range <- c(range(data$x, na.rm=TRUE), range(data$y, na.rm=TRUE)) 
  data
}

map.clip.poly <- function(data, xlim=c(NA, NA), ylim=c(NA, NA), poly=FALSE) {
  nam <- data$names
# Make sure that x & y have the same NA delimiters!
  data$x[is.na(data$y)] <- NA
  data$y[is.na(data$x)] <- NA
# >= xlim[1]
  if (!is.na(xlim[1])) {
    len_in <- length(data$x)
    len_out <- 2*len_in
    nseg <- sum(is.na(data$x)) + 1
    dd <- .C(C_map_clip_poly,
               xin=as.numeric(data$x), yin=as.numeric(data$y),
               nin=as.integer(len_in),
               xout=numeric(len_out), yout=numeric(len_out), 
               nout=as.integer(len_out),
               xlim=as.numeric(xlim[1]), inside=as.integer(1),
               poly=as.integer(poly), npoly=integer(nseg),
               NAOK=TRUE)
    data$x <- dd$xout[1:dd$nout]
    data$y <- dd$yout[1:dd$nout]
    if (!is.null(nam) && poly) nam <- rep(nam, times=dd$npoly)
  }

# <= xlim[2]
  if (!is.na(xlim[2])) {
    len_in <- length(data$x)
    len_out <- 2*len_in
    nseg <- sum(is.na(data$x)) + 1
    dd <- .C(C_map_clip_poly,
               xin=as.numeric(data$x), yin=as.numeric(data$y),
               nin=as.integer(len_in),
               xout=numeric(len_out), yout=numeric(len_out), 
               nout=as.integer(len_out),
               xlim=as.numeric(xlim[2]), inside=as.integer(-1),
               poly=as.integer(poly), npoly=integer(nseg),
               NAOK=TRUE)
    data$x <- dd$xout[1:dd$nout]
    data$y <- dd$yout[1:dd$nout]
    if (!is.null(nam) && poly) nam <- rep(nam, times=dd$npoly)
  }

# >= ylim[1]
  if (!is.na(ylim[1])) {
    len_in <- length(data$y)
    len_out <- 2*len_in
    nseg <- sum(is.na(data$y)) + 1
    dd <- .C(C_map_clip_poly,
               yin=as.numeric(data$y), xin=as.numeric(data$x),
               nin=as.integer(len_in),
               yout=numeric(len_out), xout=numeric(len_out), 
               nout=as.integer(len_out),
               ylim=as.numeric(ylim[1]), inside=as.integer(1),
               poly=as.integer(poly), npoly=integer(nseg),
               NAOK=TRUE)
    if (!is.null(nam) && poly) nam <- rep(nam, times=dd$npoly)
    data$x <- dd$xout[1:dd$nout]
    data$y <- dd$yout[1:dd$nout]
  }
 
# <= ylim[2]
  if (!is.na(ylim[2])) {
    len_in <- length(data$y)
    len_out <- 2*len_in
    nseg <- sum(is.na(data$y)) + 1
    dd <- .C(C_map_clip_poly,
               yin=as.numeric(data$y), xin=as.numeric(data$x),
               nin=as.integer(len_in),
               yout=numeric(len_out), xout=numeric(len_out), 
               nout=as.integer(len_out),
               ylim=as.numeric(ylim[2]), inside=as.integer(-1),
               poly=as.integer(poly), npoly=integer(nseg),
               NAOK=TRUE)
    data$x <- dd$xout[1:dd$nout]
    data$y <- dd$yout[1:dd$nout]
    if (!is.null(nam) && poly) nam <- rep(nam, times=dd$npoly)
  }

  if (!is.null(nam) && poly) data$names <- nam
  data$range <- c(range(data$x, na.rm=TRUE), range(data$y, na.rm=TRUE))
  data
}

# Functions for smoothing or dis-aggregating data over map regions.

# m is a map object
# z is a named vector
# res is resolution of sampling grid
# span is kernel parameter (larger = smoother)
#   span = Inf is a special case which invokes cubic spline kernel.
#   span is scaled by the map size, and is independent of res.
# result is a frame
smooth.map <- function(m, z, res = 50, span = 1/10, averages = FALSE,
                       type = c("smooth", "interp"), merge = FALSE) {
  #if(is.data.frame(z)) z = as.named.vector(z)
  if(averages) {
    # turn averages into sums
    z = z * area.map(m, names(z), sqmi=FALSE)
  }
  # sampling grid
  xlim <- range(m$x, na.rm = TRUE)
  ylim <- range(m$y, na.rm = TRUE)
  midpoints <- function(start, end, n) {
    inc <- (end - start)/n
    seq(start + inc/2, end - inc/2, length.out = n)
  }
  # 2*res is an assumption about aspect ratio (usually true)
  if(length(res) == 1) res = c(2*res, res)
  xs <- midpoints(xlim[1], xlim[2], res[1])
  ys <- midpoints(ylim[1], ylim[2], res[2])
  x <- expand.grid(x = xs, y = ys)
  if(FALSE) {
    # add centroids to the sample points
    xc = apply.polygon(m[c("x", "y")], centroid.polygon)
    # convert m into a matrix
    xc <- t(array(unlist(xc), c(2, length(xc))))
    xc = data.frame(x = xc[, 1], y = xc[, 2])
    x = rbind(x, xc)
  }
  radius = sqrt(diff(xlim)^2 + diff(ylim)^2)/2
  lambda = 1/(span*radius)^2
  #cat("lambda = ", lambda, "\n")
  cell.area = diff(xs[1:2])*diff(ys[1:2])

  r <- factor(map.where(m, x))
  if(merge) {
    # merge regions
    # merge[r] is the parent of region r
    # regions with merge[r] = NA are considered absent from the map
    # (no sample points will be placed there)
    # this can be slow on complex maps
    merge <- names(z)
    merge <- merge[match.map(m, merge)]
    names(merge) <- m$names
    levels(r) <- merge[levels(r)]
  }
  # remove points not on the map
  i <- !is.na(r)
  x <- x[i, ]
  r <- r[i]
  xo = x
  if(TRUE) {
    # kludge - drop regions with no samples
    n = table(r)
    bad = (n == 0)
    newlevels = levels(r)
    newlevels[bad] = NA
    levels(r) = newlevels
  }
  # put z in canonical order, and drop values which are not in the map
  z = z[levels(r)]
  # remove regions not named in z, or where z is NA
  bad = is.na(z)
  z = z[!bad]
  newlevels = levels(r)
  newlevels[bad] = NA
  levels(r) = newlevels
  i <- !is.na(r)
  x <- x[i, ]
  r <- r[i]
  # do all regions have sample points?
  n = table(r)
  if(any(n == 0)) stop(paste(paste(names(n)[n == 0], collapse = ", "), "have no sample points"))
  type <- match.arg(type)
  if(FALSE) {
    # code for these is in 315/util.r
    # most time is spent here
    # w <- switch(type,
    #             mass = gp.smoother(x, x, r, lambda),
    #             smooth = kr.smoother(x, x, r, lambda))
    # z = drop(z %*% w)
    # cbind(x, z = z)
  } else {
    if(type == "smooth") {
      z = kernel.smooth(x, z, xo, lambda, r)
    } else {
      z = gp.smooth(x, z, xo, lambda, r)
    }
    z = z/cell.area
    cbind(xo, z = z)
  }
}

gp.smooth <- function(x, z, xo, lambda, r) {
  # predict a function measured at locations x to new locations xo
  krr = kernel.region.region(x, r, lambda)
  white.z = solve(krr, z)
  kernel.smooth(x, white.z, xo, lambda, r, normalize = FALSE)
}

kernel.smooth <- function(x, z, xo, lambda, region = NULL, normalize = TRUE) {
  # predict a function measured at locations x to new locations xo
  if(!is.matrix(x)) dim(x) <- c(length(x), 1)
  if(!is.matrix(xo)) dim(xo) <- c(length(xo), 1)
  n = nrow(x)
  if(is.null(region)) region = 1:n
  if(length(region) < n) stop("region must have same length as x")
  region = as.integer(region)
  if(any(is.na(region))) stop("region has NAs")
  if(max(region) > length(z)) stop("not enough measurements for the number of regions")
  no = nrow(xo)
  if(normalize) {
    # divide by region sizes
    z = as.double(z/as.numeric(table(region)))
  }
  .C(C_kernel_smooth,
     as.integer(n), as.integer(ncol(x)),
     as.double(t(x)), z, as.integer(region),
     as.integer(no), as.double(t(xo)), zo = double(no),
     as.double(lambda), as.integer(normalize))$zo
}

kernel.region.region <- function(x, region, lambda) {
  if(!is.matrix(x)) dim(x) <- c(length(x), 1)
  region = as.integer(region)
  nr = max(region)
  krr = .C(C_kernel_region_region,
    as.integer(nrow(x)), as.integer(ncol(x)),
    as.double(t(x)),
    region, as.double(lambda), as.integer(nr), krr = double(nr*nr))$krr
  dim(krr) = c(nr, nr)
  krr
}
kernel.region.x <- function(x, region, z, lambda) {
  if(!is.matrix(x)) dim(x) <- c(length(x), 1)
  if(!is.matrix(z)) dim(z) <- c(length(z), 1)
  region = as.integer(region)
  nr = max(region)
  no = nrow(z)
  krx = .C(C_kernel_region_x,
    as.integer(nrow(x)), as.integer(ncol(x)),
    as.double(t(x)), region, as.integer(no), as.double(t(z)),
    as.double(lambda), as.integer(nr), krx = double(nr*no))$krx
  dim(krx) = c(nr, no)
  krx
}

#!/bin/sh
for i in *.line
do
cat $i | tr -s ' \t' '\012' | grep -n EOR | \
awk -F: '{if($1-p>m)m=$1-p;p=$1} \
	END{print NR,m}' > ${i}stats
done

for i in *.gon
do
cat $i | sed 's/^[ 	]//' | tr -s ' 	' '\012' | \
awk '{if ($0!="EOR") {nw++} else {nr++; if (nw>maxnw) maxnw=nw; nw=0}}
END{print nr,maxnw}' > ${i}stats
done

\name{internal2}
\alias{internal2}
\alias{char.to.ascii}
\alias{is.regexp}
\alias{indicators.factor}
\alias{insert}
\alias{match.map.slow}
\alias{match.map.grep}
\alias{map.poly}
\alias{map.wrap}
\alias{map.wrap.poly}
\alias{map.clip.poly}
\alias{subgroup}
\alias{gp.smooth}
\alias{kernel.smooth}
\alias{kernel.region.region}
\alias{kernel.region.x}
\alias{.map.range}
\title{Internally Required Functions}

\usage{
char.to.ascii(s)
is.regexp(s)
indicators.factor(y)
insert(x, i, v)
match.map.slow(nam, regions, warn = FALSE)
match.map.grep(nam, regions, warn = FALSE)
map.poly(database, regions = ".", exact = FALSE, xlim = NULL, ylim = NULL,
         boundary = TRUE, interior = TRUE, fill = FALSE, as.polygon = FALSE,
         namefield="name")
map.wrap(p, xlim=NULL)
map.wrap.poly(data, xlim, poly = FALSE, antarctica = -89.5)
map.clip.poly(data, xlim = c(NA, NA), ylim = c(NA, NA), poly = FALSE)
subgroup(x, i)
gp.smooth(x, z, xo, lambda, r)
kernel.smooth(x, z, xo, lambda, region = NULL, normalize = TRUE)
kernel.region.region(x, region, lambda)
kernel.region.x(x, region, z, lambda)
.map.range(new)
}

\description{
These functions are called internally and will generally not be
required by the user.
}

\keyword{internal}

\name{county.fips}
\docType{data}
\alias{county.fips}
\title{
FIPS county codes for US County Map
}
\description{
A database matching FIPS codes to maps package county and state names.
}
\usage{data(county.fips)}
\format{
A list with 2 components, namely "fips" and "polyname", containing the
FIPS number and respective state or county polygon name. Note that "fips" is represented as an integer, so any leading zero (which is part of the fips code) is not shown by default.
}
\seealso{
\code{\link{state.fips}}
}
\keyword{datasets}

\author{Tom Minka}
\name{map.text}
\alias{map.text}
\title{
  Draw a map with labeled regions
}
\description{
  Like \code{\link{map}}, but labels the regions.
}
\usage{
map.text(database, regions = ".", exact = FALSE, labels, cex = 0.75,
add = FALSE, move = FALSE, ...)
}
\arguments{
  \item{database}{character string naming a geographical database, or a list of
    \code{x}, \code{y}, and \code{names} obtained from a previous call to
    \code{map}.}
  \item{regions}{character vector that names the polygons to draw.}
  \item{exact}{If 'TRUE', only exact matches with 'regions' are selected
    for drawing.}
  \item{labels}{character vector of labels, one for each region
    selected.  Defaults to the names in the database.}
  \item{cex}{character expansion factor.}
  \item{add}{If \code{FALSE}, a map is drawn, then labels placed on top.  If
    \code{TRUE}, labels are added to the existing map.}
  \item{move}{If \code{TRUE}, labels are moved so that they don't
    overlap.  Requires the \code{mining} library (not in CRAN, contact
    tpminka@media.mit.edu).}
  \item{...}{Other arguments are the same as in \code{\link{map}}.}
}
\value{
  If \code{add = FALSE}, a map is drawn by calling \code{\link{map}}.
  Then the label for each region is placed at the centroid of the region
  polygon.

  The return value is a map object, as from \code{\link{map}}.
}
\examples{
map.text("world", "ira")    # iran and iraq
map.text("state", "penn")
map.text("county", "penn")  # Pennsylvania counties
map.text("county", "new jersey")  # New Jersey counties
}
\keyword{hplot}

\name{iso.expand}
\alias{sov.expand}
\alias{iso.expand}
\alias{iso.alpha}
\title{
  Identify countries by ISO 3166 codes (2 or 3 letters) or by Sovereignty.
}
\description{
  This data set and the simple look-up functions allow to build lists of counrtries for the world map.
  }
\usage{
	iso.expand(a, regex=TRUE)
	sov.expand(sov, regex=TRUE)
        iso.alpha(x, n=2)
}
\arguments{
  \item{a}{A vector of ISO codes. All elements should have the same length, either 2 or 3 letters. Not case sensitive.}
  \item{sov}{A vector of country names. The result is a list of all countries that fall under their sovereignty. Case sensitive, must fit completeley.}
  \item{regex}{If TRUE (default), the return vector has the same length as the input (\code{a} or \code{sov}), but the entries may be regular expressions.
If FALSE, the result is a vector of polygon names. This may be more readable, but the return vector may be longer than the input.}
  \item{x}{Vector of country names, may include colons.}
  \item{n}{An integer identitying which ISO code is required. Allowed values are 2 and 3.}
}
\details{
  The ISO 3166-1 standard identifies countries by a 2 and 3 letter codes.
\code{iso.expand} translates these codes into the country names as used by 
the \code{world} data base. \code{iso.alpha} does the reverse.  
Some countries have different ISO codes for different regions 
(e.g. China:Hong Kong has ISO code HK). In such cases, \code{iso.alpha} will 
return the main code, but \code{iso.expand} will return a regular expression 
that excludes some parts.
}
\value{
  \code{iso.expand} returns vector of country names. When used as input for \code{map} it will plot all the countries as identified either by their sovereignty or by ISO codes. If \code{regex=FALSE} the length of the vector may be shorter or longer than the input. If \code{regex=TRUE}, the results are concatenated in regular expressions. This format is less readable, but can be used as input e.g. for \code{match.map}.
\code{iso.alpha} always returns a vector of the same length as the input,  containing the 2- or 3-letter codes.
}
\section{NOTE}{
These functions use regular expressions and the results will often not work well with \code{map(...,exact=TRUE)}.
}
\seealso{
\code{\link{match.map}},\code{\link{map.text}},\code{\link{iso3166}}
}
\references{
\url{https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2}
}
\examples{
# France and all its overseas departments, territories etc.
sov.expand("France") # France and all its overseas departments, territories etc.

# Canary Islands are not included in map("Spain")
iso.expand("ES")
map(regions=sov.expand("Spain"))

# draw a map with ISO codes as labels:
wm <- map("world", fill=TRUE, col=0, xlim=c(-10,40), ylim=c(30,60))
# take out islands, but you loose e.g. UK, New Zealand, small island states
nam <- grep(":", wm$names, inv=TRUE, val=TRUE)
# ad ISO codes as label
map.text(wm, regions=nam, label=iso.alpha(nam), col=2, exact=TRUE, add=TRUE)
}

\name{world.cities}
\docType{data}
\alias{world.cities}
\title{
Database of world cities
}

\description{
This database is primarily of world cities of population greater than
about 40,000.  Also included are capital cities of any population size, and
many smaller towns.
}
\usage{data(world.cities)}
\format{
A list with 6 components, namely "name", "country.etc", "pop", "lat",
"long", and "capital", containing the city name, the country name,
approximate population (as at January 2006), latitude, longitude and
capital status indication (0 for non-capital, 1 for capital, 2 for China
Municipalities, and 3 for China Provincial capitals)
}
\source{
The data were originally obtained from Stefan Helders' website (http://www.world-gazetteer.com), which no longer exists. There are no recent updates available.
}
\section{NOTE}{
Some of the country names and city names may be out of date.  Please send
any corrections to the package maintainer.
}
\seealso{
\code{\link{map.cities}}
}
\keyword{datasets}

\name{map.scale}
\alias{map.scale}
\title{
Add Scale to Existing Unprojected Map
}
\description{
Adds a scale to an existing map, both as a ratio and a distance gauge.
}
\section{NOTE}{
  This function is meaningful only if no projection= has been specified
  in the call to map().
}
\usage{
map.scale(x, y, relwidth = 0.15, metric = TRUE, ratio = TRUE, \dots)
}
\arguments{
\item{x}{
Horizontal location of left end of distance gauge.  If not specified,
this will be taken to be near the lower left corner of the map.
}
\item{y}{
Vertical location of left end of distance gauge.  If not specified,
this will be taken to be near the lower left corner of the map.
}
\item{relwidth}{
Proportion of width of display to be used for the scale.  The default is
0.15 (15\%).
}
\item{metric}{
  If \code{TRUE}, the distance gauge will be in km, otherwise miles.
}
\item{ratio}{
  If \code{FALSE}, the scale ratio of the map is not displayed.
}
\item{...}{
Further plotting parameters may be specified as for the command text().
}}
\value{
The exact calculated scale is returned.
}
\section{Side Effects}{
A scale is added to the currently displayed map. This takes the form of
an approximate 1:n scale (containing 2-3 significant digits), above a
distance gauge which is reasonably accurate for the latitude at which it
appears.  The circumference at the given latitude is interpolated from a
radius of 6356.78 km at the pole and 6378.16 km at the equator.
}
\details{
The scale is calculated from the displayed graph's plotting parameters,
and the latitude of the location at which the distance gauge will be
displayed.
}
\seealso{
\code{\link{map.axes}}
}
\examples{
map("world", "China")
map.scale()
}
\keyword{hplot}

\name{polygon}
\alias{apply.polygon}
\alias{area.polygon}
\alias{centroid.polygon}
\alias{as.matrix.polygon}
\alias{closed.polygon}
\alias{in.polygon}
\alias{in.one.polygon}
\alias{num.polygons}
\alias{sub.polygon}
\title{Polygon functions}
\usage{
area.polygon(p)
centroid.polygon(p)
\method{as.matrix}{polygon}(x, ...)
closed.polygon(p)
in.one.polygon(p, x)
in.polygon(p, x)
num.polygons(p)
sub.polygon(p, i)
}
\description{
These functions are called internally and will generally not be
required by the user.
}
\keyword{internal}

\name{state.vbm}
\docType{data}
\alias{state.vbm}
\alias{state.vbmMapEnv}
\alias{state.vbm.center}
\title{
United States State Visibility Base Map
}
\usage{
data(state.vbmMapEnv)
data(state.vbm.center)
}
\description{

This database produces a map of the states of the United States
mainland. The Visibility Base Map was created by Mark Monmonier
to provide simplified state shapes with sufficient areas to allow
annotations in even the small states.

\code{state.vbm.center} are coordinates of the state centers
for annotation purposes. The states are alphabetically ordered, in the same order as the map. So state names can be matched via e.g. \code{map(state.vbm, plot=FALSE)$name}.
}
\format{
The data file is merely a character string which
specifies the name of an environment variable which contains the
base location of the binary files used by the map drawing functions.
This environment variable (\code{R_MAP_DATA_DIR} for the datasets in the
maps package) is set at package load time \emph{if it does not
already exist}.  Hence setting the environment variable before loading
the package can override the default location of the binary datasets.
}
\seealso{
\code{\link{map}}.
}
\examples{
map('state.vbm', fill = TRUE, col = palette())
}
\references{
Richard A. Becker, and Allan R. Wilks,
"Maps in S",
\emph{AT&T Bell Laboratories Statistics Research Report [93.2], 1993.}

Richard A. Becker, and Allan R. Wilks,
"Constructing a Geographical Database",
\emph{AT&T Bell Laboratories Statistics Research Report [95.2], 1995.}

Mark Monmonier and George Schnell,
"The Study of Population",
\emph{Elements, Patterns, Processes. Charles E. Merrill. Columbus, OH. 1982.}
}
\keyword{datasets}

\name{legacy_world}
\docType{data}
\alias{legacy_world}
\alias{legacy_worldMapEnv}
\alias{legacy_world2}
\alias{legacy_world2MapEnv}
\title{ Legacy low resolution World Map. DEPRECATED!}
\usage{data(worldMapEnv)}
\description{
 This world database comes from a thinned cleaned-up version of the CIA World Data Bank II data and contains approximately 30,000 points representing the world coastlines and national boundaries approximatly around 1990.
}
\format{
The data file is merely a character string which
specifies the name of an environment variable which contains the
base location of the binary files used by the map drawing functions.
This environment variable (\code{R_MAP_DATA_DIR} for the datasets in the
maps package) is set at package load time \emph{if it does not
already exist}.  Hence setting the environment variable before loading
the package can override the default location of the binary datasets.

During a transition period (\code{maps} version 3.0) it is possible to switch to the old legacy map using the function \code{world.legacy(TRUE)} or by setting the setting the environment variable R_MAP_DATA_LEGACY=TRUE prior to loading the package.

In future releases, this dataset will be removed completely. It will however remain accessible via the \code{mapdata} package (as of version 2.3.0), where it is called \code{worldLores}

}
\source{
The CIA World Data Bank II is currently (2015) still available from 
\url{https://www.evl.uic.edu/pape/data/WDB/}.
}
\seealso{
\code{\link{map}},\code{\link{world.legacy}},\code{\link{world}}
}
\examples{
## DEPRECATED:
# map('legacy_world', fill = TRUE, col = 1:10)
## Use mapdata::worldLores instead:
if (requireNamespace("mapdata", quietly=TRUE) && packageVersion("mapdata") >= "2.3") {
  map("mapdata::worldLores", fill = TRUE, col = 1:10)
}
}
\references{
Richard A. Becker, and Allan R. Wilks,
"Maps in S",
\emph{AT\&T Bell Laboratories Statistics Research Report [93.2], 1993.}

Richard A. Becker, and Allan R. Wilks,
"Constructing a Geographical Database",
\emph{AT\&T Bell Laboratories Statistics Research Report [95.2], 1995.}
}
\keyword{datasets}

\name{state.carto}
\docType{data}
\alias{state.carto}
\alias{state.cartoMapEnv}
\alias{state.carto.center}
\title{
United States State Population Cartogram Map
}
\usage{
data(stateMapEnv)
data(state.carto.center)
}
\description{

This database produces a cartogram of the states of the United States
mainland based on CartoDraw, roughly proportional to population (see
references).

\code{state.carto.center} are coordinates of the state centers
for annotation purposes.
}
\format{
The data file is merely a character string which
specifies the name of an environment variable which contains the
base location of the binary files used by the map drawing functions.
This environment variable (\code{R_MAP_DATA_DIR} for the datasets in the
maps package) is set at package load time \emph{if it does not
already exist}.  Hence setting the environment variable before loading
the package can override the default location of the binary datasets.
}
\seealso{
\code{\link{map}}.
}
\examples{
map('state.carto', fill = TRUE, col = palette())
}
\references{
Richard A. Becker, and Allan R. Wilks,
"Maps in S",
\emph{AT&T Bell Laboratories Statistics Research Report [93.2], 1993.}

Richard A. Becker, and Allan R. Wilks,
"Constructing a Geographical Database",
\emph{AT&T Bell Laboratories Statistics Research Report [95.2], 1995.}

CartoDraw,
\url{http://www.computer.org/csdl/trans/tg/2004/01/v0095-abs.html}
}
\keyword{datasets}

\name{us.cities}
\docType{data}
\alias{us.cities}
\title{
Database of US cities
}

\description{
This database is of us cities of population greater than about 40,000.
Also included are state capitals of any population size.
}
\format{
A list with 6 components, namely "name", "country.etc", "pop",  "lat",
"long", and "capital", containing the city name, the state abbreviation,
approximate population (as at January 2006), latitude, longitude and
capital status indication (0 for non-capital, 1 for capital, 2 for state
capital.
}
\section{NOTE}{
Some of the city names may be out of date.  Please send
any corrections to the package maintainer.
}
\seealso{
\code{\link{map.cities}}
}
\keyword{datasets}

\name{italy}
\docType{data}
\alias{italy}
\alias{italyMapEnv}
\title{
Italy Map
}
\usage{data(italyMapEnv)}
\description{
This italy database comes from the NUTS III (Tertiary Administrative
Units of the European Community) database of the United Nations
Environment Programme (UNEP) GRID-Geneva data sets. These were prepared
around 1989, and so may be somewhat out of date.

Users of data sets supplied through UNEP/GRID are requested to
incorporate in output products and reports acknowledgements to the
originator of the data and to the fact that they were acquired through
UNEP/GRID. Appropriate wording may be "UNESCO (1987) through
UNEP/GRID-Geneva". 
}
\format{
The data file is merely a character string which
specifies the name of an environment variable which contains the
base location of the binary files used by the map drawing functions.
This environment variable (\code{R_MAP_DATA_DIR} for the datasets in the
maps package) is set at package load time \emph{if it does not
already exist}. Hence setting the environment variable before loading
the package can override the default location of the binary datasets.
}
\seealso{
\code{\link{map}}
}
\examples{
map('italy', fill = TRUE, col = 1:10)
}
\references{
Richard A. Becker, and Allan R. Wilks,
"Maps in S",
\emph{AT&T Bell Laboratories Statistics Research Report [93.2], 1993.}

Richard A. Becker, and Allan R. Wilks,
"Constructing a Geographical Database",
\emph{AT&T Bell Laboratories Statistics Research Report [95.2], 1995.}
}
\keyword{datasets}

\author{Tom Minka}
\name{map.where}
\alias{map.where}
\title{
  Locate points on a map
}
\description{
  Returns the region names containing given locations.
}
\usage{
map.where(database = "world", x, y, ...)
}
\arguments{
  \item{database}{character string naming a geographical database, or a list of
    \code{x}, \code{y}, and \code{names}.
    See the documentation for \link{map} for more details.}
  \item{x}{vector of longitudes.}
  \item{y}{vector of latitudes.}
  \item{...}{Options for \code{SpatialPolygons2map}, only used if \code{database} is of type \code{SpatialPolygonsDataFrame}}.
}
\value{
  A list of character strings, naming the map region that each
  (longitude, latitude) pair falls into.
}
\seealso{\code{in.polygon}}
\note{
  For points close to a border (polygon boundary), the result may be wrong if the
  resolution of the database is insufficient.
  This function may also give erroneous results if the database contains
  enclaves. For instance, a point in San Marino may also be identified 
  as being in Italy.
}
\examples{
# NYC
map.where("state", -73.8, 41)
# Auckland
map.where("nz", 174.6, -36.92)
# find both in the world
map.where(x = c(174.6, -73.8), y = c(-36.92, 41))
# with a map object:
m = map("state", "new york", fill = TRUE, plot = FALSE)
map.where(m, -73.8, 41)
}
\keyword{iplot}

\name{iso3166}
\alias{iso3166}
\docType{data}
\title{
  ISO 3166 country codes (2 or 3 letters) and sovereignty.
}
\description{
  This data set lists all ISO3166 country codes and the sovereignty for each country in the list. Some entries are regular expressions.
}
\details{
  The ISO 3166-1 standard identifies countries by a 2 and 3 letter codes. This table listst these for all countries on the world map. This data set also serves as basis for the function iso.expand() and its siblings.}
\format{
A data frame with 5 columns: "a2", "a3", "name", "mapname", "sovereignty". These contain the 2- and 3-letter ISO code, the official name,  the (possibly shorter) name used in the map data base, and the sovereign country.
}
\section{NOTE}{
Some countries have different ISO codes for some regions. To deal with such particular cases, the "mapname" column may sometimes contain (perl-style) regular expressions rather than simply a country name. For instance, "FI" has mapname "Finland(?!:Aland)", because the Aland islands have a different ISO code. Other codes may appear in two rows if certain parts of countries are not written with the main country as base name. Usually, that is for compatibility with the legacy world data base. 
}

\seealso{
\code{\link{iso.expand}}
}
\references{
\url{https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2}
}
\keyword{datasets}

\name{ozone}
\docType{data}
\alias{ozone}
\alias{votes.repub}
\alias{unemp}
\title{Sample datasets}
\usage{
data(ozone)
data(unemp)
data(votes.repub)
}
\description{Datasets used to illustrate map functions.

\code{ozone} contains the median of daily maxima ozone concentration in 41 US cities for June 1974 through August 1974. Concentrations are in parts per billion (ppb).

\code{unemp} Has population and unemployment percentage for US counties.

\code{votes.repub} contains the percentage republican votes in the 1900 election.
}
\references{
Cleveland, W.S., Kleiner, B., McRae, J.E., Warner, J.L., and Pasceri, P.E. ,
"The Analysis of Ground-Level Ozone Data from New Jersey, New York, Connecticut, and Massachusetts: Data Quality Assessment and Temporal and Geographical Properties", \emph{Bell Laboratories Memorandum, 1975}.


}
\keyword{datasets}

\name{Spatial2map}
\alias{Spatial2map}
\alias{SpatialPolygons2map}
\alias{SpatialLines2map}
\alias{sf2map}
\title{
  Read sf, SpatialPolygons and SpatialLines objects
}
\description{
  These functions transform some classes provided by the packages \code{sp} and \code{sf} into a simple list that can be used by map().
}
\usage{
	SpatialPolygons2map(database, namefield=NULL)
	SpatialLines2map(database, namefield=NULL)
        sf2map(database, namefield="name")
}
\arguments{
  \item{database}{A \code{SpatialPolygons}, \code{SpatialLines} or \code{sf} object.}
  \item{namefield}{The name of a data column in \code{database} to be used for naming the polygons (or lines). If it is a vector of names, these are all used and separated by a colon ':'. Not case sensitive. So if the database contains columns that only differ by case, you get a warning and namefield is not used at all.}
}
\value{
A list with four components: \code{x, y, names, range}, similar to the return value of \code{map()}. This data can be used as a database for \code{map()}. The lines and polygons are separated by NA.}
\details{The 'map' list object only preserves co-ordinates and polygon names. All other information available in the original data is lost. For instance, plotting order for enclaves may be wrong, resulting in invisible polygons when setting \code{fill=TRUE}.

The option \code{namefield} is only taken into account if \code{database} is class \code{Spatial[]DataFrame}. \code{namefield} may be a vector of column names, e.g. to get polygons named as 'country:state'.}
\seealso{
\code{\link{map}},\code{\link[sp]{SpatialPolygons}} (in the \code{sp} library).
}

\name{canada.cities}
\docType{data}
\alias{canada.cities}
\title{
Database of Canadian cities
}

\description{
This database is of Canadian cities of population greater than about 1,000.
Also included are province capitals of any population size.
}
\format{
A list with 6 components, namely "name", "country.etc", "pop", "lat",
"long", and "capital", containing the city name, the province
abbreviation, approximate population (as at January 2006), latitude,
longitude and capital status indication (0 for non-capital, 1 for
capital, 2 for provincial
 
capital.
}
\section{NOTE}{
Some of the city names may be out of date.  Please send
any corrections to the package maintainer.
}
\seealso{
\code{\link{map.cities}}
}
\keyword{datasets}

\author{Tom Minka}
\name{smooth.map}
\alias{smooth.map}
\title{
  Smooth out aggregated data
}
\description{
  Increases the resolution of data aggregated over map regions,
  by either smoothing or interpolation.
  Also fills in missing values.
}
\usage{
smooth.map(m, z, res = 50, span = 1/10, averages = FALSE, type = c("smooth",
"interp"), merge = FALSE)
}
\arguments{
  \item{m}{a map object}
  \item{z}{a named vector}
  \item{res}{a vector of length two, specifying the resolution of the
    sampling grid in
    each dimension.  If a single number, it is taken as the vertical
    resolution, with double taken as the horizontal resolution.}
  \item{span}{kernel parameter (larger = smoother).
    \code{span = Inf} is a special case which invokes the cubic spline kernel.
    \code{span} is automatically scaled by the map size, and is
    independent of \code{res}.}
  \item{averages}{If \code{TRUE}, the values in \code{z} are interpreted
    as averages over the regions.  Otherwise they are interpreted as totals.}
  \item{type}{see details.}
  \item{merge}{If \code{TRUE}, a region named in \code{z} includes
    all matching regions in the map (according to
    \code{\link{match.map}}).
    If \code{FALSE}, a region named in
    \code{z} is assumed to refer to exactly one region on the map.}
}
\details{
  For \code{type = "smooth"}, the region totals are first converted
  into point measurements on the
  sampling grid, by dividing the total for a region among all sample
  points inside it.  Then it is a regular kernel smoothing 
  problem.  Note that the region totals are not preserved.
  
  The prediction \eqn{z_o}{zo} for
  location \eqn{x_o}{xo} (a vector) is the average of \code{z} for
  nearby sample points:
  
  \deqn{z_o = \frac{\sum_x k(x, x_o) z(x)}{\sum_x k(x, x_o)}}{%
    zo = (sum_x k(x, xo) z(x))/(sum_x k(x, xo))}
  \deqn{k(x, x_o) = exp(-\lambda ||x - x_o||^2)}{%
    k(x, xo) = exp(-lambda ||x - xo||^2)}
  \eqn{\lambda}{lambda} is determined from \code{span}.
  Note that \eqn{x_o}{xo} is over the same sampling grid as
  \eqn{x}{x}, but \eqn{z_o}{zo} is not necessarily the same as
  \eqn{z(x_o)}{z(xo)}.
  
  For \code{type = "interp"}, the region totals are preserved by the
  higher-resolution function.
  The function is assumed to come from a
  Gaussian process with kernel \eqn{k}{k}.  The measurement \code{z[r]}
  is assumed to be the sum of the function over the discrete sample
  points inside region \code{r}.
  This leads to a simple formula for the covariance matrix of \code{z}
  and the cross-covariance between \code{zo} and \code{z}.
  The prediction is the cross-covariance times the inverse covariance
  times \code{z}.  Unlike Tobler's method, the predictions are not
  constrained to live within the original data range,
  so there tends to be "ringing" effects.

  See the references for more details.
}
\value{
  A data frame with columns \code{x}, \code{y}, and \code{z}
  giving the smoothed value \code{z} for locations (x, y).
  Currently the (x, y) values form a grid, but this is not
  guaranteed in the future.
}
\references{
  W.F. Eddy and A. Mockus. 
  An example of the estimation and display of a smoothly varying
  function of time and space - the incidence of disease mumps.
  \emph{Journal of the American Society for Information Science},
  45(9):686-693, 1994.
  \url{https://web.eecs.utk.edu/~audris/papers/jasis.pdf}
  
  W. R. Tobler.  Smooth pycnophylactic interpolation for
  geographical regions.  \emph{Journal of the American Statistical
  Association} 74:519-530, 1979.
}
\examples{
# compare to the example for match.map
data(state, package = "datasets")
data(votes.repub)
z = votes.repub[, "1900"]
m = map("state", fill = TRUE, plot = FALSE)
# use a small span to fill in, but not smooth, the data
# increase the resolution to get better results
fit = smooth.map(m, z, span = 1/100, merge = TRUE, ave = TRUE)
mat = tapply(fit$z, fit[1:2], mean)
gray.colors <- function(n) gray(rev(0:(n - 1))/n)
par(bg = "blue")
filled.contour(mat, color.palette = gray.colors, nlev = 32, asp = 1)
# another way to visualize:
image(mat, col = gray.colors(100))

# for a higher degree of smoothing:
# fit = smooth.map(m, z, merge = TRUE, ave = TRUE)
# interpolation, state averages are preserved:
# fit = smooth.map(m, z, merge = TRUE, ave = TRUE, type = "interp")
}
\keyword{dplot}

\author{Tom Minka}
\name{area.map}
\alias{area.map}
\title{
  Area of projected map regions
}
\usage{
area.map(m, regions = ".", sqmi=TRUE, ...)
}
\description{Computes the areas of regions in a projected map.}
\arguments{
  \item{m}{a map object containing named polygons (created with
  \code{fill = TRUE}).}
  \item{regions}{a character vector naming one of more regions, as in
    \code{\link{map}}.}
  \item{sqmi}{If \code{TRUE}, measure area in square miles.  Otherwise
    keep the units of \code{m}.}
  \item{...}{additional arguments to \code{\link{match.map}}}
}
\details{
  The area of each matching region in the map is computed, and
  regions which match the same element of \code{regions} have their
  areas combined.  Each region is assumed planar, with vertices
  specified by the \code{x} and \code{y} components of the map object.

  The correct use of this function is to first use \code{\link{map}} to
  create polygons and project the coordinates onto a plane, then apply
  \code{area.map} to compute the area of the projected regions.
  If the projection is area-preserving (such as \code{albers}),
  then these areas will match the area on the globe, up to a
  constant.  To get an absolute area in square miles, the \code{sqmi}
  option will scale the result, depending on the projection.

  The coordinates from \code{\link{map}} are affected by 
  its \code{resolution} argument, so use \code{resolution=0} for the most
  accurate areas.
}
\section{NOTE}{
  The \code{sqmi} option assumes the coordinates have been projected
  with the \link[mapproj]{mapproject} function.
}
\value{
  a named vector of region areas.
}
\seealso{\code{area.polygon}, \code{apply.polygon}}
\examples{
# because the projection is rectangular, these are not true areas on the globe.
m = map("state", fill = TRUE, plot = FALSE)
area.map(m)
area.map(m, ".*dakota")
area.map(m, c("North Dakota", "South Dakota"))

if(require(mapproj)) {
  # true areas on the globe
  m = map("state", proj="bonne", param=45, fill=TRUE, plot=FALSE)
  # North Dakota is listed as 70,704 square miles
  area.map(m, "North Dakota")
}
}
\keyword{dplot}

\name{nz}
\docType{data}
\alias{nz}
\alias{nzMapEnv}
\title{
New Zealand Basic Map
}
\usage{data(nzMapEnv)}
\description{
This database produce a map of New Zealand at a basic level of
detail.  The `"nz"' database includes the 3 main Islands and 19 smaller
coastal islands.
}
\format{
The data file is merely a character string which
specifies the name of an environment variable which contains the
base location of the binary files used by the map drawing functions.
This environment variable (\code{R_MAP_DATA_DIR} for the datasets in the
maps package) is set at package load time \emph{if it does not
already exist}.  Hence setting the environment variable before loading
the package can override the default location of the binary datasets.
}
\seealso{
\code{\link{map}}
}
\examples{
map('nz')
map('nz', xlim = c(166, 179), ylim = c(-48, -34))
}
\references{
Richard A. Becker, and Allan R. Wilks,
"Maps in S",
\emph{AT&T Bell Laboratories Statistics Research Report [93.2], 1993.}

Richard A. Becker, and Allan R. Wilks,
"Constructing a Geographical Database",
\emph{AT&T Bell Laboratories Statistics Research Report [95.2], 1995.}
}
\keyword{datasets}

\name{world2}
\docType{data}
\alias{world2}
\alias{world2MapEnv}
\title{
Pacific Centric Low resolution World Map
}
\usage{data(world2MapEnv)}
\description{
This is an alternative version of the \code{world} database 
based on latitudes [0, 360), which then has the Pacific
Ocean in the centre of the map.
}
\section{NOTE}{
This data set is in fact largely obsolete. Often the same (more general) result can be obtained by using wrapping:

map("world", wrap=c(0,360))

This will also work fine with \code{fill=TRUE} or any other appropriate longitude interval (e.g. \code{c(-90,270)}).

However, \code{world2} is useful when setting \code{xlim} to an interval crossing the 180 meridian.
}
\format{
The data file is merely a character string which
specifies the name of an environment variable which contains the
base location of the binary files used by the map drawing functions.
This environment variable (\code{R_MAP_DATA_DIR_WORLD} for the datasets in the
maps package) is set at package load time \emph{if it does not
already exist}.  Hence setting the environment variable before loading
the package can override the default location of the binary datasets.
}
\source{
The public domain Natural Earth data set is available from \url{https://www.naturalearthdata.com}.
}
\seealso{
\code{\link{map}}, \code{\link{world}}
}
\examples{
map('world2', xlim = c(100, 300))
map.axes()
# xlim is performed before wrapping:
map('world', wrap=c(0,360), xlim = c(100, 300))
# so to emulate "world2":
ww2 <- map('world', wrap=c(0,360), plot=FALSE, fill=TRUE)
map(ww2, xlim = c(100, 300), fill=TRUE)
}
\references{
Richard A. Becker, and Allan R. Wilks,
"Maps in S",
\emph{AT&T Bell Laboratories Statistics Research Report [93.2], 1993.}

Richard A. Becker, and Allan R. Wilks,
"Constructing a Geographical Database",
\emph{AT&T Bell Laboratories Statistics Research Report [95.2], 1995.}
}
\keyword{datasets}

\name{world}
\docType{data}
\alias{world}
\alias{worldMapEnv}
\title{
Low (mid) resolution World Map
}
\usage{data(worldMapEnv)}
\description{
This world map (updated in 2013) is imported from the public domain Natural Earth project (the 1:50m resolution version). It replaces a much older version based on the CIA World Data Bank II data.
The old legacy data is still available in the package \code{mapdata} (v2.3.0).
}
\format{
The data file is merely a character string which
specifies the name of an environment variable which contains the
base location of the binary files used by the map drawing functions.
This environment variable (\code{R_MAP_DATA_DIR_WORLD}) is set at package load time \emph{if it does not
already exist}.  Hence setting the environment variable before loading
the package can override the default location of the binary datasets.

}
\source{
The Natural Earth data set is in the public domain and available from \url{https://www.naturalearthdata.com}.
}
\details{
As of version 3.1, the \code{world} database no longer contains any lakes. These have been moved to a separate database called \code{lakes}.
The legacy world map (dating from around 1990) has been removed from the package and is now available from the \code{mapdata} package in two different resolutions (worldHires and worldLores).
}
\seealso{
\code{\link{map}},\code{\link{lakes}}
}
\examples{
# notice how some polygons extend beyond the [-180,180] interval:
map('world', fill = TRUE, col = 1:10)
# if you wrap at [-180,180], you also can get a clean closure of Antarctica
map('world', fill = TRUE, col = 1:10, wrap=c(-180,180) )
}
\references{
Richard A. Becker, and Allan R. Wilks,
"Maps in S",
\emph{AT&T Bell Laboratories Statistics Research Report [93.2], 1993.}

Richard A. Becker, and Allan R. Wilks,
"Constructing a Geographical Database",
\emph{AT&T Bell Laboratories Statistics Research Report [95.2], 1995.}
}
\keyword{datasets}

\name{lakes}
\docType{data}
\alias{lakes}
\alias{lakesMapEnv}
\title{
World lakes database
}
\usage{data(lakesMapEnv)}
\description{

This database contains a selection of large lakes (and islands within) taken from the Natural Earth 1:50m map, the same data source as the (v3.0) world map. The lake boundaries are consistent with the 'world' database.
}
\format{
The data file is merely a character string which
specifies the name of an environment variable which contains the
base location of the binary files used by the map drawing functions.
This environment variable (\code{R_MAP_DATA_DIR} for the datasets in the
maps package) is set at package load time \emph{if it does not
already exist}.  Hence setting the environment variable before loading
the package can override the default location of the binary datasets.
}
\seealso{
\code{\link{map}}.
}
\examples{
map('world')
map('lakes', add=TRUE, fill=TRUE, col='white', boundary='black')
}
\source{
The data in this data base is derived from the public domain GIS project Natural Earth, the file "ne_50m_lakes". The Natural Earth data set is available from \url{https://www.naturalearthdata.com}.}
\references{
\emph{Natural Earth project}
\url{https://www.naturalearthdata.com}
}
\keyword{datasets}

\author{Tom Minka}
\name{match.map}
\alias{match.map}
\title{
Index map regions
}
\description{
Assigns an index to each map region, useful for map coloring.
}
\usage{
match.map(database, regions, exact = FALSE, warn = TRUE)
}
\arguments{
  \item{database}{
    character string naming a geographical database, or a map object.
    See the documentation for \code{\link{map}} for more details.
  }
  \item{regions}{
    a vector of names, or more generally regular expressions
    to match against the map region names.
  }
  \item{exact}{If \code{TRUE}, only exact matches with \code{regions}
    are considered.  Otherwise each element of \code{regions} is assumed
    to be a regular expression.
    Matches are always case-insensitive.}
  \item{warn}{If \code{TRUE}, a warning is printed when an element of
    \code{regions} matches nothing in the map.}
}
\value{
  Returns an integer vector giving an index to each region in the database.
  The index is the index of the string in \code{regions} which matches the
  region name.  Matching is done as in \code{\link{map}}.
  More specifically, all regions \code{r} whose name matches 
  \code{regions[i]} will have index \code{i}.
  Unmatched regions will have index \code{NA}.
  Overlapping matches cause an error.

  This behavior differs from \code{\link{pmatch}} because a single entry
  in \code{regions} may match several entries in the map.
}
\seealso{\code{\link{grep}}}
\examples{
# filled map showing Republican vote in 1900
# (figure 6 in the reference)
data(state, package = "datasets")
data(votes.repub)
state.to.map <- match.map("state", state.name)
x <- votes.repub[state.to.map, "1900"]
gray.colors <- function(n) gray(rev(0:(n - 1))/n)
color <- gray.colors(100)[floor(x)]
map("state", fill = TRUE, col = color); map("state", add = TRUE)
}
\references{
Richard A. Becker, and Allan R. Wilks,
"Maps in S",
\emph{AT&T Bell Laboratories Statistics Research Report, 1991.}
}
\keyword{dplot}

\name{internal}
\alias{internal}
\alias{makepoly}
\alias{mapgetg}
\alias{mapgetl}
\alias{mapname}
\alias{mapthin}
\alias{maptype}
\title{Internally Required Functions}

\usage{
makepoly(xy, gonsize, keep)
mapgetg(database, gons, fill, xlim, ylim)
mapgetl(database, lines, xlim, ylim, fill)
mapname(database, patterns, exact)
mapthin(xy, delta, symmetric)
maptype(database)
}

\description{
These functions are called internally and will generally not be
required by the user.
}

\keyword{internal}

\name{state}
\docType{data}
\alias{state}
\alias{stateMapEnv}
\title{
United States State Boundaries Map
}
\usage{data(stateMapEnv)}
\description{

This database produces a map of the states of the United States mainland
generated from US Department of the Census data (see the reference).
}
\format{
The data file is merely a character string which
specifies the name of an environment variable which contains the
base location of the binary files used by the map drawing functions.
This environment variable (\code{R_MAP_DATA_DIR} for the datasets in the
maps package) is set at package load time \emph{if it does not
already exist}.  Hence setting the environment variable before loading
the package can override the default location of the binary datasets.
}
\seealso{
\code{\link{map}}.
}
\examples{
map('state', fill = TRUE, col = palette())
}
\references{
Richard A. Becker, and Allan R. Wilks,
"Maps in S",
\emph{AT&T Bell Laboratories Statistics Research Report [93.2], 1993.}

Richard A. Becker, and Allan R. Wilks,
"Constructing a Geographical Database",
\emph{AT&T Bell Laboratories Statistics Research Report [95.2], 1995.}

US Department of Commerce, Census Bureau,
\emph{County Boundary File,}
computer tape, available from Customer Services,
Bureau of the Census, Washingdon DC 20233.
}
\keyword{datasets}

\name{map.cities}
\alias{map.cities}
\title{
Add Cities to Existing Map
}
\usage{
map.cities(x = world.cities, country = "", label = NULL, minpop = 0,
maxpop = Inf, capitals = 0, cex = par("cex"), projection = FALSE,
parameters = NULL, orientation = NULL, pch = 1, \dots)
}
\description{
  Adds city locations and (optionally) names to an existing map using a
  specified database.
}
\arguments{
\item{x}{
Name of database. See \code{\link{world.cities}} to determine the structure of
the database.
}
\item{country}{
If the string country is specified, limit the displayed cities to be
from within the specified country, province or state (depending on how
the database has been constructed).
}
\item{label}{
If \code{TRUE}, label all cities.  If \code{NULL},
the cities will be labelled unless there are 20 or more.
}
\item{minpop}{
The minimum value of population below which a particular city will not
be shown.
}
\item{maxpop}{
The maximum value of population above which a particular city will not
be shown.
}
\item{capitals}{
Selection of capitals-only display. Capitals may be 1 (country capital),
2 (provincial, state, or regional capital) or 3 (local capital). See
\code{\link{world.cities}} for further information.
}
\item{cex}{
The value of cex acts to override the current value of character size
expansion.
}
\item{projection}{
Boolean or character value.  If \code{FALSE} (the default), no
projection is assumed, if \code{TRUE}, the previous projection is used,
otherwise a character string that names a map projection to use.
See \code{\link[mapproj]{mapproject}} (in the \code{mapproj} library).
}
\item{parameters}{
numeric vector of parameters for use with the \code{projection}
argument.  This argument is optional only in the sense that certain
projections do not require additional parameters.  If a projection
does require additional parameters, these must be given in the
\code{parameters} argument.
}
\item{orientation}{
a vector \code{c(latitude, longitude, rotation)} describing where
the map should be centered and a clockwise rotation (in degrees)
about this center.
}
\item{pch}{
plotting character to use for marking city location. See
\code{\link{points}} for options.
}
\item{...}{
Further plotting parameters may be specified as for the commands
\code{\link{points}} and \code{\link{text}}.
}}
\value{
No value is returned from map.cities.
}
\section{Side Effects}{
All cities within the boundaries of the plot containing the current map
are added to the plot.  Note that it is possible that the boundaries of
the plot exceed the boundaries of the map requested, and so more cities
than were expected might be shown.
}
\details{
The database is searched for all cities matching the specified criteria
and fitting within the limits of the plot currently displayed.  The
default database is of all cities that have a population greater than a
certain threshold or which are capital cities of a country or island
territory.  The threshold varies from country to country, but in
general is no higher than about 40,000.  The data were originally obtained from Stefan Helders' website (http://www.world-gazetteer.com), which no longer exists. There are no recent updates available.

There are three supplied databases, world.cities (the default), us.cities
and canada.cities.  The latter two, which need to be made available by
using a \code{'data()'} call, include the state or province name with the
city name (thanks to John Woodruff \email{jpwoodruff@irisinternet.net}
for the state and province information).

Note that if the underlying map is "Pacific-centric", i.e. longitudes
exceed 180 degrees, and a projection is used, then the map.cities data
must be transformed appropriately.
}
\seealso{
\code{\link{world.cities}}, \code{\link{canada.cities}},
\code{\link{us.cities}}
}
\examples{
map("world", "China")
map.cities(country = "China", capitals = 2)
map("state", "New Jersey")
data(us.cities)
map.cities(us.cities, country="NJ")
}
\keyword{hplot}

\author{Tom Minka}
\name{identify.map}
\alias{identify.map}
\title{
  Identify regions on a map
}
\description{
  Identifies the map regions clicked by the user.
}
\usage{
\method{identify}{map}(x, n = 1, index = FALSE, ...)
}
\arguments{
  \item{x}{a map object containing named polygons.}
  \item{n}{the number of clicks to wait for.}
  \item{index}{If TRUE, returns the index of the polygon, rather than
    its name.}
  \item{...}{additional arguments passed to \code{\link{identify.default}}.}
}
\details{
  The current algorithm is somewhat crude --- selects the region whose
  centroid is closest to the click.  A more sophisticated approach would
  use \code{\link{map.where}}.
}
\value{
  a character vector of length \code{n}, naming the selected regions.
}
\seealso{\code{\link{identify}}, \code{\link{map.where}}}
\examples{
identify(map("state", fill = TRUE, col = 0))
if(require(mapproj))
  identify(map("world", proj = "lagrange", fill = TRUE, col = 0, wrap=c(-180,180,-90)))
}
\keyword{iplot}

\name{map.axes}
\alias{map.axes}
\title{
  Draw Axes on Geographical Maps
}
\description{Draws a set of axes on an existing map.}
\usage{
map.axes(...)
}
\arguments{
  \item{...}{Extra arguments passed to \code{axis} or \code{box}.}
}
\section{Side Effects}{
x- and y-axes are drawn for the currently displayed map.  These will
display in longitude and latitude (if no projection= has been
specified in the map() call).
}
\examples{
map("state")
map.axes(cex.axis=0.8)
}
\keyword{hplot}

\name{map}
\alias{map}
\title{
  Draw Geographical Maps
}
\description{Draw lines and polygons as specified by a map database.}
\usage{
map(database = "world", regions = ".", exact = FALSE, boundary = TRUE,
  interior = TRUE, projection = "", parameters = NULL, orientation = NULL,
  fill = FALSE, col = 1, plot = TRUE, add = FALSE, namesonly = FALSE,
  xlim = NULL, ylim = NULL, wrap = FALSE, resolution = if (plot) 1 else 0,
  type = "l", bg = par("bg"), mar = c(4.1, 4.1, par("mar")[3], 0.1),
  myborder = 0.01, namefield="name", lforce="n", ...)
}
\arguments{
  \item{database}{
    character string naming a geographical database, a list of
    \code{x}, \code{y}, and \code{names} obtained from a previous call to
    \code{map} or a spatial object of class \code{SpatialPolygons} or \code{SpatialLines}.
    The string choices include a \code{\link{world}} map,
    three USA databases (\code{\link{usa}}, \code{\link{state}},
    \code{\link{county}}), and more (type \code{help(package='maps')} to see the package index). If the required database is in a different package that has not been attached, the string may be started with "packagename::". The
    location of the map databases may be overridden by setting the
    \code{R_MAP_DATA_DIR} environment variable.
  }
  \item{regions}{
    character vector that names the polygons to draw.
    Each database is composed of a collection of polygons, and each polygon has
    a unique name.
    When a region is composed of more than one polygon, the individual polygons
    have the name of the region, followed by a colon and a qualifier,
    as in \code{michigan:north} and \code{michigan:south}.
    Each element of \code{regions} is matched
    against the polygon names in the database and, according to
    \code{exact}, a subset is selected for drawing. The regions may also be 
    defined using (perl) regular expressions. This makes it possible to use 
    'negative' expressions like \code{"Norway(?!:Svalbard)"}, which means Norway
    and all islands except Svalbard. All entries are case insensitive.
    The default selects all polygons in the database.
  }
  \item{exact}{
    If \code{TRUE}, only exact matches with \code{regions} are
    selected for drawing.
    If \code{FALSE}, each element of \code{regions} is matched
    as a regular expression
    against the polygon names in the database and all matches are selected
    for drawing.
  }
  \item{boundary}{
    If \code{FALSE}, boundary segments are not drawn.
    A boundary segment is a line segment of the map that bounds only
    one of the polygons to be drawn.
    This argument is ignored if \code{fill} is \code{TRUE}.
  }
  \item{interior}{
    If \code{FALSE}, interior segments are not drawn.
    An interior segment is a line segment of the map that bounds two
    of the polygons to be drawn.
    This argument is ignored if \code{fill} is \code{TRUE}.
  }
  \item{projection}{
    character string that names a map projection to use.
    See \code{\link[mapproj]{mapproject}} (in the \code{mapproj} library).
    The default is to use a rectangular projection with the aspect ratio
    chosen so that longitude and latitude scales are equivalent at the
    center of the picture.
  }
  \item{parameters}{
    numeric vector of parameters for use with the
    \code{projection} argument.
    This argument is optional only in the sense that certain
    projections do not require additional parameters.
    If a projection does require additional parameters, these
    must be given in the \code{parameters} argument.
  }
  \item{orientation}{
    a vector \code{c(latitude, longitude, rotation)} describing where
    the map should be centered and a clockwise rotation (in degrees)
    about this center.
  }
  \item{fill}{
    logical flag that says whether to draw lines or fill areas.
    If \code{FALSE}, the lines bounding each region will be drawn (but
    only once, for interior lines).
    If \code{TRUE}, each region will be filled using colors from the
    \code{col =} argument, and bounding lines are drawn by default using \code{par("fg")}.
    To hide the bounding lines, use \code{border=NA} (see \code{...}).
  }
  \item{col}{
    vector of colors.
    If \code{fill} is \code{FALSE}, the first color is used for plotting
    all lines, and any other colors are ignored.
    Otherwise, the colors are matched one-one with the polygons
    that get selected by the \code{region} argument (and are reused
    cyclically, if necessary). If \code{fill = TRUE}, the default boundary line colour 
    is given by \code{par("fg")}. To change this, you can use the \code{border} 
    argument (see '...').
    A color of \code{NA} causes the corresponding region to be deleted
    from the list of polygons to be drawn.
    Polygon colors are assigned \emph{after} polygons are deleted due to values
    of the \code{xlim} and \code{ylim} arguments.
  }
  \item{plot}{
    logical flag that specifies whether plotting
    should be done.
    If \code{plot} is \code{TRUE} the return value of \code{map} will
    not be printed automatically .
  }
  \item{add}{
    logical flag that specifies whether to add to the
    current plot.
    If \code{FALSE}, a new plot is begun, and a new coordinate
    system is set up.
  }
  \item{namesonly}{
    If \code{TRUE}, the return value will be a character vector of
    the names of the selected polygons.  See the Value section below.
  }
  \item{xlim}{
    two element numeric
    vector giving a range of longitudes, expressed
    in degrees, to which drawing
    should be restricted.
    Longitude is measured in degrees east of Greenwich, so that, in particular,
    locations in the USA have negative longitude.
    If \code{fill = TRUE}, polygons selected by \code{region} must be entirely inside
    the \code{xlim} range.
    The default value of this argument spans the entire longitude range
    of the \code{database}.
  }
  \item{ylim}{
    two element 
    numeric vector giving a range of latitudes,
    expressed in degrees, to which drawing
    should be restricted.
    Latitude is measured in degrees north of the
    equator, so that, in particular,
    locations in the USA have positive latitude.
    If \code{fill = TRUE}, polygons selected by \code{region} must be entirely inside
    the \code{ylim} range.
    The default value of this argument spans the entire latitude range
    of the \code{database}.
  }
  \item{wrap}{Boolean or a numeric vector. If TRUE, lines that cross too far across the map
    (due to a strange projection) are omitted. If wrap is a vector of length 2 or more, it is interpreted as the longitude range to be used for a global map, e.g. \code{c(-180,180)} or \code{c(0,360)}. This wrapping even works when \code{fill=TRUE} and is performed before any projection (so the range must always be in degrees). However, the wrapping is performed \emph{after} \code{xlim, ylim} are applied, so these options should probably never be combined. If there is a third component, this signifies the latitude at which Antarctica will be "closed". The default value is -89.9. Special values are \code{NA} (don't draw Antarctica at all) and 0 (draw the line at the latitude of the extremal points, not at a fixed lower latitude). }
  \item{resolution}{
    number that specifies the resolution with which
    to draw the map.
    Resolution 0 is the full resolution of the database.
    Otherwise, just before polylines are plotted they are thinned:
    roughly speaking, successive points on the polyline that are
    within \code{resolution} device pixels of one another are collapsed
    to a single point (see the Reference for further details).
    Thinning is not performed if \code{plot = FALSE} or when polygons are drawn (\code{fill = TRUE} or \code{database} is a list of polygons).
  }
  \item{type}{
    character string that controls drawing of the map.
    Aside from the default \code{type = "l"}, the value \code{type = "n"}
    can be used
    to set up the coordinate system and projection for a map that will
    be added to in later calls.
  }
  \item{bg}{
    background color.
  }
  \item{mar}{
    margins, as in \code{\link{par}}.  Defaults allow for map.axes().
  }
  \item{myborder}{
    scalar or vector of length 2 specifying the porportion of the plot
    to add to the defined or computed limits as borders.
  }
  \item{namefield}{
    A vector of column names to be used as region name if \code{database} is a \code{SpatialPolygonsDataFrame}. Ignored in all other cases.
  }
  \item{lforce}{Limit enforcement. Only taken into account if \code{xlim} and/or \code{ylim} are defined and (for "s" and "l") a projection is used. Possible values are \code{"n"} (none) \code{"e"} (exact),\code{"s"} (small),\code{"l"} (large). 
If \code{lforce="e"}, map data is restricted exactly to the given limits prior to projection. This option affects the output value of the map, not only the plot window.
If \code{lforce="s"}, the four corners defined by \code{xlim} and \code{ylim} are projected and the plot window is restricted to the largest rectangle inside this region. 
\code{"l"} will result in a larger rectangle that includes the four corners. However, the parts that fall outside the original limits may not be complete (only polygons that partially fall inside the boundaries are included). This will also work with \code{fill=TRUE}. In this case the output value of the data is not changed, only the plot window is affected. These options are only useful for some projections.}
  \item{...}{Extra arguments passed to \code{polygon} or \code{lines}. Of particular interest may be the options \code{border} and\code{lty} that control the color and line type of the polygon borders when \code{fill = TRUE}.}
}
\details{
The simplest form of use of this function is:
\preformatted{map(mymap)}
where \code{mymap} is the returned value from a previous call to
\code{map()}.
}
\value{
  If \code{plot = TRUE}, a plot is made where 
  the polygons selected from \code{database}, through the
  \code{regions}, \code{xlim}, and \code{ylim} arguments, are outlined
  (\code{fill} is \code{FALSE}) or filled (\code{fill} is \code{TRUE})
  with the colors in \code{col}.  

  The return value is a list with
  \code{x}, \code{y}, \code{range}, and \code{names} components.  
  This object can be used as a \code{database} for successive calls
  to \code{map} and functions.
  If \code{fill} is \code{FALSE}, the \code{x} and \code{y} vectors are
  the coordinates of successive polylines, separated by \code{NA}s.  If
  \code{fill} is \code{TRUE}, the \code{x} and \code{y} vectors have
  coordinates of successive polygons, again separated by \code{NA}s.
  Thus the return value can be handed directly to \code{lines} or
  \code{polygon}, as appropriate.

  When \code{namesonly} is \code{TRUE}, only the names component is returned.

  After a call to \code{map} for which the \code{projection} argument was
  specified there will be a global variable \code{.Last.projection}
  containing information about the projection used.
  This will be consulted in subsequent calls to \code{map} which use
  \code{projection = ''}.
}
\examples{
map()	# low resolution map of the world
map(wrap = c(0,360), fill = TRUE, col = 2) # pacific-centered map of the world
map(wrap = c(0, 360, NA), fill = TRUE, col = 2) # idem, without Antarctica
map('usa')	# national boundaries
map('county', 'new jersey')	# county map of New Jersey
map('state', region = c('new york', 'new jersey', 'penn'))	# map of three states
map("state", ".*dakota", myborder = 0)	# map of the dakotas
map.axes()				# show the effect of myborder = 0
if(require(mapproj))
  map('state', proj = 'bonne', param = 45)	# Bonne equal-area projection of states

# names of the San Juan islands in Washington state
map('county', 'washington,san', names = TRUE, plot = FALSE)

# national boundaries in one linetype, states in another
# (figure 5 in the reference)
map("state", interior = FALSE)
map("state", boundary = FALSE, lty = 2, add = TRUE)

# plot the ozone data on a base map
# (figure 4 in the reference)
data(ozone)
map("state", xlim = range(ozone$x), ylim = range(ozone$y))
text(ozone$x, ozone$y, ozone$median)
box()
if(require(mapproj)) {	# mapproj is used for  projection="polyconic"
  # color US county map by 2009 unemployment rate
  # match counties to map using FIPS county codes
  # Based on J's solution to the "Choropleth Challenge"
  # http://blog.revolutionanalytics.com/2009/11/choropleth-challenge-result.html

  # load data
  # unemp includes data for some counties not on the "lower 48 states" county
  # map, such as those in Alaska, Hawaii, Puerto Rico, and some tiny Virginia
  #  cities
  data(unemp)
  data(county.fips)

  # define color buckets
  colors = c("#F1EEF6", "#D4B9DA", "#C994C7", "#DF65B0", "#DD1C77", "#980043")
  unemp$colorBuckets <- as.numeric(cut(unemp$unemp, c(0, 2, 4, 6, 8, 10, 100)))
  leg.txt <- c("<2\%", "2-4\%", "4-6\%", "6-8\%", "8-10\%", ">10\%")

  # align data with map definitions by (partial) matching state,county
  # names, which include multiple polygons for some counties
  cnty.fips <- county.fips$fips[match(map("county", plot=FALSE)$names,
    county.fips$polyname)]
  colorsmatched <- unemp$colorBuckets [match(cnty.fips, unemp$fips)]

  # draw map
  map("county", col = colors[colorsmatched], fill = TRUE, resolution = 0,
    lty = 0, projection = "polyconic")
  map("state", col = "white", fill = FALSE, add = TRUE, lty = 1, lwd = 0.2,
    projection="polyconic")
  title("unemployment by county, 2009")
  legend("topright", leg.txt, horiz = TRUE, fill = colors)

  # Choropleth Challenge example, based on J's solution, see:
  # http://blog.revolutionanalytics.com/2009/11/choropleth-challenge-result.html
  # To see the faint county boundaries, use RGui menu:  File/SaveAs/PDF
}
}
\seealso{
  \code{\link{map.text}}, \code{\link{map.axes}},
  \code{\link{map.scale}}, \code{\link[mapproj]{map.grid}} (in the \code{mapproj} library),
  \code{\link[graphics]{polygon}}, \code{\link{SpatialPolygons2map}}
}
\references{
  Richard A. Becker, and Allan R. Wilks,
  "Maps in S",
  \emph{AT&T Bell Laboratories Statistics Research Report [93.2], 1993.}
  \url{https://web.archive.org/web/20050825145143/http://www.research.att.com/areas/stat/doc/93.2.ps}

  Richard A. Becker, and Allan R. Wilks,
  "Constructing a Geographical Database",
  \emph{AT&T Bell Laboratories Statistics Research Report [95.2], 1995.}
  \url{https://web.archive.org/web/20050825145143/http://www.research.att.com/areas/stat/doc/95.2.ps}
}
\keyword{hplot}
% Converted by Sd2Rd version 0.3-3.
